// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________
#ifndef __GOOEY_CARET_H__
#define __GOOEY_CARET_H__


#include "glgooey/core/Core.h"


// *************************************************************************************************
namespace Gooey
{


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! \brief The caret is defined by the \e dot (the position of the caret within the text) and the 
//! \e marka selection index. If these two
//! indices are the same then nothing is selected.
class Caret
{
    // ---------------------------------------------------------------------------------------------
    //  Construction / Destruction
    // ---------------------------------------------------------------------------------------------
public:
    //! \brief constructor can be given the index at which to initially position the dot and mark
    explicit Caret(size_t anIndex = 0) : dot_(anIndex), mark_(anIndex) {}


    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    //! Returns the index which determines the caret position
    size_t dot() const { return dot_; }
    
    //! Returns the index which determines the selection together with dotIndex()
    size_t mark() const { return mark_; }

    //! Returns true if anything is selected
    bool isAnythingSelected() const { return dot_ != mark_; }
    
    //! Returns the starting index of the selection
    size_t selectionStart() const { return std::min(dot_, mark_); }

    //! Returns the ending index of the selection
    size_t selectionEnd() const { return std::max(dot_, mark_); }
    
    //! returns the number of characters encompassed by the current selection
    size_t selectionSize() const { return selectionEnd() - selectionStart(); }
    
    //! \brief Puts the dot at the passed in index, taking the mark with it, effectively deleting 
    //! the selection
    Caret& setDot(size_t anIndex) { dot_ = mark_ = anIndex; dotMoved(dot()); return *this; }
    
    //! \brief Moves the dot to the passed in position, leaving the mark where it is
    Caret& moveDot(size_t anIndex) { dot_ = anIndex; dotMoved(dot()); return *this; }
    
    //! Removes the selection without moving the caret
    Caret& removeSelection() { return setDot(dot()); }
    
    //! Selects everything between the passed in start and end indices leaving the dot at the end
    Caret& select(size_t aStartIndex, size_t anEndIndex) { return setDot(aStartIndex).moveDot(anEndIndex); }

    
    // ---------------------------------------------------------------------------------------------
    //  Signals
    // ---------------------------------------------------------------------------------------------
public:
    //! \brief This signal is emitted whenever the position of the dot changes
    Signal1<size_t> dotMoved;
    
    
    // ---------------------------------------------------------------------------------------------
    //  Implementation
    // ---------------------------------------------------------------------------------------------
private:
    size_t dot_;   //!< The index which determines the caret position
    size_t mark_;  //!< The index which determines the selection together with the dot
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<




// *************************************************************************************************
} // namespace


#endif
