// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________

#include <algorithm>
#include <numeric>

#include "glgooey/ComplexGridLayouter.h"

#include "glgooey/core/StandardException.h"
#include "glgooey/Window.h"


// *************************************************************************************************
namespace Gooey
{


REGISTER_CLASS(ComplexGridLayouter);


// *************************************************************************************************
namespace 
{


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! \brief Helper class for the complex grid layouter
//!
//! Computes the widths(heights) of columns(rows) using a vector of sizes specified on 
//! construction. The result is a vector of sizes containing one entry for each column(row) 
//! required by the layouter
class SlotSizeData
{
    // ---------------------------------------------------------------------------------------------
    //  Construction / Destruction
    // ---------------------------------------------------------------------------------------------
public:
    //! \brief Constructor requires the specified sizes, the amount of available space and the 
    //! number of slots (i.e. rows or columns) for which sizes are required
    SlotSizeData(const std::vector<float>& sizes, float anAvailableSize, size_t aNumberOfSlots);
    
    
    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    //! Returns an iterator referencing the begin of the size vector
    std::vector<float>::const_iterator beginOfSizeVector() const { return sizes_.begin(); }
    
    //! Returns an iterator referencing the end of the size vector
    std::vector<float>::const_iterator endOfSizeVector() const { return sizes_.end(); }


    // ---------------------------------------------------------------------------------------------
    //  Implementation
    // ---------------------------------------------------------------------------------------------
private:
    //! Computes the relevant sizes - This is the first thing called in the constructor
    void computeSizes();
    
    //! Returns the total required size used by the layouter
    float requiredSize() const;
    
    //! Returns the number of non-zero specified sizes_
    size_t numberOfFixedSlots() const;
    
    //! Returns the number zero-valued specified sizes_
    size_t numberOfVariableSlots() const;

    //! Returns the number of slots for which a size is required, but has not been specified
    size_t numberOfUnsetSlots() const;
    
    //! \brief Updates the total size with the passed in size
    //! \note The passed in size will be cropped necessary.
    void updateTotalSize(float& aSize);
    
    //! \brief Returns the size of the variable slots. This is the left over space divided by 
    //! the number of unset slots
    float variableSlotSize() const;

    //! Sets the variable slot widths to their actual width
    void setVariableSlotSizes();
    
    //! Adds any additional slot sizes_ which may be required to make up the number of slots
    void addAdditionalSlotSizes();

    const float availableSize_;        //!< The amount of space available to the layouter
    const size_t numberOfSlots_;       //!< The number of slots these sizes have to fill
    
    std::vector<float> sizes_;         //!< The final sizes to be used for the windows controlled by this layouter
    float  requiredSize_;              //!< The total size used by the layouter (cannot exceed the available size)
    size_t numberOfVariableSlots_;     //!< The number of zero entries in sizes_
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
SlotSizeData::SlotSizeData(const std::vector<float>& sizes, float anAvailableSize, size_t aNumberOfSlots) :
    availableSize_(anAvailableSize),
    numberOfSlots_(aNumberOfSlots),
    sizes_(sizes),
    requiredSize_(0.0f),
    numberOfVariableSlots_(0)
{
    computeSizes();

    if(numberOfUnsetSlots() > 0)
    {
        setVariableSlotSizes();
        addAdditionalSlotSizes();
    }
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void 
SlotSizeData::computeSizes()
{
    for(std::vector<float>::iterator it = sizes_.begin(); it != sizes_.end(); ++it)
    {
        float& sizeEntry = *it;
        if(sizeEntry == 0.0f)
            ++numberOfVariableSlots_;
        else
            updateTotalSize(sizeEntry);
    }
    ASSERT(sizes_.size() >= numberOfVariableSlots_);
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
float 
SlotSizeData::requiredSize() const 
{ 
    return requiredSize_; 
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
size_t 
SlotSizeData::numberOfFixedSlots() const 
{ 
    return sizes_.size() - numberOfVariableSlots_; 
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
size_t 
SlotSizeData::numberOfVariableSlots() const 
{ 
    return numberOfVariableSlots_; 
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
size_t 
SlotSizeData::numberOfUnsetSlots() const
{
    return (numberOfSlots_ > numberOfFixedSlots()) ? numberOfSlots_ - numberOfFixedSlots() : 0;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void 
SlotSizeData::updateTotalSize(float& aSize)
{
    const float oldTotalSize = requiredSize_;
    requiredSize_ += aSize;
    if(requiredSize_ > availableSize_)
    {
        aSize = availableSize_ - oldTotalSize;
        requiredSize_ = availableSize_;
    }
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
float 
SlotSizeData::variableSlotSize() const
{
    float width = 0.0f;
    if(requiredSize() < availableSize_)
    {
        const float remainingSize = availableSize_ - requiredSize();
        width = remainingSize / static_cast<float>(numberOfUnsetSlots());
    }
    return width;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void 
SlotSizeData::setVariableSlotSizes()
{
    std::replace(sizes_.begin(), sizes_.end(), 0.0f, variableSlotSize());
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void 
SlotSizeData::addAdditionalSlotSizes()
{
    sizes_.resize(numberOfSlots_, variableSlotSize());
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<




// *************************************************************************************************
} // anonymous namespace






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
ComplexGridLayouter&
ComplexGridLayouter::setColumnWidths(const std::vector<float>& widths)
{
    columnWidths_ = widths;
    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
ComplexGridLayouter&
ComplexGridLayouter::setRowHeights(const std::vector<float>& heights)
{
    rowHeights_ = heights;
    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
float
ComplexGridLayouter::columnWidth(size_t anIndex) const
{
    return (anIndex < columnWidths_.size()) ?  columnWidths_[anIndex] : 0.0f;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
float
ComplexGridLayouter::rowHeight(size_t anIndex) const
{
    return (anIndex < rowHeights_.size()) ? rowHeights_[anIndex] : 0.0f;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
BaseLayouter&
ComplexGridLayouter::addWindow(Window* window)
{
    addWindow(window, ComplexGridCellInfo());
    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
BaseLayouter&
ComplexGridLayouter::addWindow(Window* window, const LayoutInfo& inf)
{
    ASSERT(inf.type() == "ComplexGridCellInfo");

    ComplexGridCellInfo info = (const ComplexGridCellInfo&)(inf);

    Cell cell(window, info);
    cells_.push_back(cell);

    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
BaseLayouter&
ComplexGridLayouter::removeWindow(Window* window)
{
    std::list<Cell>::iterator it;
    for(it = cells_.begin(); it != cells_.end(); ++it)
    {
        if( (*it).window_ == window ) break;
    }
    if(it != cells_.end()) cells_.erase(it);

    return *this;

}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
BaseLayouter&
ComplexGridLayouter::arrangeWithin(Window* parentWindow)
{
    const float availableWidth  = parentWindow->size().x() - (2.0f * borderSize_.x());
    const float availableHeight = parentWindow->size().y() - (2.0f * borderSize_.y());

    const Core::Vector2 offset = parentWindow->position() + borderSize_;

    const size_t numWindows = (size_t)(cells_.size());

    if(numWindows > 0)
    {
        calculateBlocks();

        SlotSizeData columnSizeData(columnWidths_, availableWidth, numCols_);
        SlotSizeData rowHeightData(rowHeights_, availableHeight, numRows_);

        std::map<size_t, Block>::iterator it;
        for(it = blockMap_.begin(); it != blockMap_.end(); ++it)
        {
            const Block& block = (*it).second;

            if(block.windowStart)
            {
                SizeIterator xStartIt = columnSizeData.beginOfSizeVector();
                std::advance(xStartIt, block.col);

                SizeIterator xEndIt = xStartIt;
                std::advance(xEndIt, block.hSpan);

                SizeIterator yStartIt = rowHeightData.beginOfSizeVector();
                std::advance(yStartIt, block.row);

                SizeIterator yEndIt = yStartIt;
                std::advance(yEndIt, block.vSpan);



                const float x = std::accumulate(columnSizeData.beginOfSizeVector(),  xStartIt, 0.0f) + block.hSpace;
                const float y = std::accumulate(rowHeightData.beginOfSizeVector(), yStartIt, 0.0f) + block.vSpace;

                const float childWidth  = std::accumulate(xStartIt, xEndIt, 0.0f) - (2.0f * block.hSpace);
                const float childHeight = std::accumulate(yStartIt, yEndIt, 0.0f) - (2.0f * block.vSpace);

                block.window->setPosition(Core::Vector2(x, y) + offset);
                block.window->setSize(Core::Vector2(childWidth, childHeight));
            }
        }
    }

    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void
ComplexGridLayouter::calculateBlocks()
{
    numCols_ = 0;
    numRows_ = 0;

    size_t nextCol = 0;
    size_t nextRow = 0;

    blockMap_.clear();

    std::list<Cell>::iterator it;
    for(it = cells_.begin(); it != cells_.end(); ++it)
    {
        const ComplexGridCellInfo& info = (*it).info_;

        if ( info.columnIndex != ComplexGridCellInfo::relativeToPrevious )
            nextCol = info.columnIndex;

        if ( info.rowIndex != ComplexGridCellInfo::relativeToPrevious )
            nextRow = info.rowIndex;

        while( blockMap_.find(nextRow*256 + nextCol) != blockMap_.end() ) ++nextCol;

        size_t hSpan = info.horizontalSpan;
        size_t vSpan = info.verticalSpan;


        for( unsigned i = 0; i != hSpan; ++i )
        {
            for( unsigned j = 0; j != vSpan; ++j )
            {
                Block block;
                block.windowStart = ( i == 0 && j == 0 );
                block.col = nextCol + i;
                block.row = nextRow + j;
                block.hSpan = hSpan;
                block.vSpan = vSpan;
                block.hSpace = info.horizontalSpacing;
                block.vSpace = info.verticalSpacing;
                block.window = (*it).window_;

                addBlockToMap(block);
            }
        }

        if ( nextCol + hSpan > numCols_ )  numCols_ = nextCol + hSpan;
        if ( nextRow + vSpan > numRows_ )  numRows_ = nextRow + vSpan;

        nextCol += hSpan;
    }
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void
ComplexGridLayouter::serialize(Core::Archive& anArchive)
{
    BaseLayouter::serialize(anArchive);    
    anArchive 
        & TAGGED_OBJECT(borderSize) 
        & TAGGED_OBJECT(columnWidths) 
        & TAGGED_OBJECT(rowHeights)
    ;    
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<








// **************************************************************************************************
//                            ComplexGridCellInfo
// **************************************************************************************************




// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
std::string ComplexGridCellInfo::type_ = "ComplexGridCellInfo";
const size_t ComplexGridCellInfo::relativeToPrevious = INT_MAX;
                                  //  std::numeric_limits<size_t>::max(); (MSVC can't handle this!!)
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
ComplexGridCellInfo::ComplexGridCellInfo() :
    horizontalSpan(1),
    verticalSpan(1),
    columnIndex(relativeToPrevious),
    rowIndex(relativeToPrevious),
    horizontalSpacing(3.0f),
    verticalSpacing(3.0f)
{
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// *************************************************************************************************
} // namespace Gooey
