// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________

#include "glgooey/core/Core.h"

#include "glgooey/EditPanel.h"

#include "glgooey/WindowManager.h"
#include "glgooey/ScrollBar.h"
#include "glgooey/SolidSurfaceDesc.h"
#include "glgooey/core/Rectangle.h"
#include "glgooey/core/StandardException.h"


// **************************************************************************************************
namespace Gooey
{


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
EditPanel::EditPanel(Window* aParent, const std::string& aText) : 
    EditWindow(aParent, aText, true),
    _verticalScrollBar(0)
{
    returnPressed.connect(this, &EditPanel::onReturnPressed);

    _verticalScrollBar = new ScrollBar(this, Core::vertical, Core::Vector2(1, 1), size().y());

    (*_verticalScrollBar)
        .setMinimum(0)
        .setMaximum(1)
        .setValue(0)
        ;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
EditPanel::~EditPanel()
{
    delete _verticalScrollBar;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
EditPanel& 
EditPanel::onReturnPressed()
{
    insertCharacter('\n');
    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
float 
EditPanel::lineHeight() const
{
    return renderDesc().textDesc().lineHeight();
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
EditPanel& 
EditPanel::positionScrollBar()
{       
    if(size().y() >= numberOfLines() * lineHeight())
    {      
        _verticalScrollBar->hide();        
    }
    else
    {
        _verticalScrollBar->show();
        
        setScrollBarParameters();

        const float barSize = _verticalScrollBar->width();
        _verticalScrollBar->setPosition(position() + Core::Vector2(size().x() - barSize, 0.0f));
        _verticalScrollBar->setSize(Core::Vector2(barSize, size().y()));
        
    }

    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void 
EditPanel::setScrollBarParameters()
{
    const int visibleAmount = static_cast<int>(heightOfVisibleContents());
    (*_verticalScrollBar)
        .setVisibleAmount(visibleAmount)
        .setUnitIncrement(static_cast<int>(lineHeight()))
        .setBlockIncrement(visibleAmount - static_cast<int>(lineHeight()))
        ;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
float 
EditPanel::heightOfAllContents() const
{
    return (numberOfLines() * lineHeight());
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
float 
EditPanel::heightOfVisibleContents() const
{
    return std::min(size().y(), heightOfAllContents());
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void 
EditPanel::onTextChanged()
{
    EditWindow::onTextChanged();
    
    if(numberOfLines() > numberOfVisibleLines())
        _verticalScrollBar->setMaximum(static_cast<int>(heightOfAllContents() - lineHeight()));
    else
        _verticalScrollBar->setMaximum(1);
        
    positionScrollBar();
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void 
EditPanel::onResize()
{
    EditWindow::onResize();
    positionScrollBar();
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void 
EditPanel::onMove()
{
    EditWindow::onMove();
    positionScrollBar();
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
bool 
EditPanel::doesHierarchyContain(Window* aWindow) const
{
    return Window::doesHierarchyContain(aWindow) || _verticalScrollBar->doesHierarchyContain(aWindow);
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
Window* 
EditPanel::windowBelow(const Core::Vector2& p)
{
    Window* ret = Window::windowBelow(p);

    if(ret != 0)
    {
        Window* clientWindow = _verticalScrollBar->windowBelow(p);
        if(clientWindow != 0)
            ret = clientWindow;
    }

    return ret;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
Window& 
EditPanel::draw()
{
    EditWindow::draw();    
    _verticalScrollBar->render();

    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
Core::Rectangle EditPanel::drawRectangle() const
{
    const Core::Vector2 topLeft = position();
    Core::Vector2 bottomRight = position() + size();
    if(_verticalScrollBar->isVisible()) bottomRight = bottomRight - Core::Vector2(_verticalScrollBar->width(), 0);
    return Core::Rectangle(topLeft, bottomRight);
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
size_t 
EditPanel::firstVisibleLine() const
{
    return (_verticalScrollBar->unitIncrement() > 0) ?
            (_verticalScrollBar->value() / _verticalScrollBar->unitIncrement()) :
            0;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
EditWindow& 
EditPanel::onCaretMoved(size_t pos)
{
    EditWindow::onCaretMoved(pos);
    
    const size_t lineWithCaret = caretLine();
    const size_t lastVisibleLine = firstVisibleLine() + numberOfVisibleLines() - 1;
    
    if(firstVisibleLine() > lineWithCaret)
        _verticalScrollBar->setValue(lineWithCaret * _verticalScrollBar->unitIncrement());
    else if(lineWithCaret > lastVisibleLine)
        _verticalScrollBar->setValue((lineWithCaret - numberOfVisibleLines() + 1) * _verticalScrollBar->unitIncrement());
    
    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
size_t 
EditPanel::numberOfVisibleLines() const
{
    return static_cast<size_t>(size().y() / lineHeight());
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<




// *************************************************************************************************
} // namespace



