// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________

#include "glgooey/EditWindow.h"

#include "glgooey/core/Utils.h"
#include "glgooey/core/Rectangle.h"

#include "glgooey/Renderer.h"
#include "glgooey/WindowManager.h"
#include "glgooey/KeyMessage.h"
#include "glgooey/PropertyScheme.h"
#include "glgooey/TimeManager.h"


// *************************************************************************************************
namespace Gooey
{

REGISTER_CLASS(EditWindowDesc);



// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! Mouse listener for the edit window
class EditWindowMouseListener : public MouseListener
{
    // ---------------------------------------------------------------------------------------------
    //  Xstruction
    // ---------------------------------------------------------------------------------------------
public:
    //! Construction rquires the edit window
    EditWindowMouseListener(EditWindow& anEditWindow) : editWindow_(anEditWindow) {}

    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    //! react to mouse clicks
    virtual bool clicked(const MouseClickMessage& aMessage)
    {
        bool isHandled = false;
        editWindow_.idealCaretXPos_ = -1.0f;
        if(isLeftClick(aMessage))
        {
            WindowManager::instance().setMouseCapture(&editWindow_);
            if(editWindow_.windowRectangle().surrounds(aMessage.position()))
            {
                editWindow_.caret().setDot(editWindow_.hitCharacter(aMessage.position()));
            }
            isHandled = true;
        }
        else if(isLeftRelease(aMessage))
        {
            if(WindowManager::instance().captureWindow() == &editWindow_)
            {
                WindowManager::instance().releaseMouseCapture();
            }
            isHandled = true;
        }
        return isHandled;
    }

    //! react to mouse movement
    virtual bool moved(const MouseMessage& aMessage)
    {
        bool isHandled = false;
        Caret& caret = editWindow_.caret();
        if(WindowManager::instance().captureWindow() == &editWindow_)
        {
            if(editWindow_.windowRectangle().surrounds(aMessage.position()))
            {
                caret.moveDot(editWindow_.hitCharacter(aMessage.position()));
            }
            else
            {
                if(aMessage.position().x() <= editWindow_.position().x())
                {
                    if(caret.dot() > 0) caret.moveDot(caret.dot() - 1);
                }
                else if(aMessage.position().x() >= editWindow_.position().x() + editWindow_.size().x())
                {
                    if(caret.dot() < editWindow_.text().length()) caret.moveDot(caret.dot() + 1);
                }
            }
            isHandled = true;
        }
        return isHandled;
    }

    // ---------------------------------------------------------------------------------------------
    //  Implementation
    // ---------------------------------------------------------------------------------------------
private:
    bool isLeftClick(const MouseClickMessage& aMessage) const
    {
        return
            (aMessage.button() == MouseClickMessage::left) &&
            ((aMessage.event() == MouseClickMessage::pressed) ||
             (aMessage.event() == MouseClickMessage::doubleClicked));
    }

    bool isLeftRelease(const MouseClickMessage& aMessage) const
    {
        return
            (aMessage.button() == MouseClickMessage::left) &&
             (aMessage.event() == MouseClickMessage::released);
    }


    EditWindow& editWindow_;

    // Copying and assignment are not allowed
    EditWindowMouseListener(const EditWindowMouseListener& anEditWindowMouseListener);
    EditWindowMouseListener& operator = (const EditWindowMouseListener& anEditWindowMouseListener);
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<










// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
EditWindow::EditWindow(Window* aParent, const std::string& aText, bool multiline) :
    Window(aParent, aText),
    caret_(aText.length()),
    renderDesc_(0),
    mouseListener_(0),
    xOffset_(0.0f),
    isMultiline_(multiline),
    idealCaretXPos_(-1.0f)
{
    mouseListener_ = new EditWindowMouseListener(*this);
    addMessageListener(mouseListener_);
    addMessageListener(this);
    
    caret().dotMoved.connect(this, &EditWindow::onCaretMoved);

    if(aText != "") onTextChanged();
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
EditWindow::~EditWindow()
{
    removeMessageListener(mouseListener_);
    delete mouseListener_;
    removeMessageListener(this);
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
EditWindow&
EditWindow::doBackspace()
{
    if(caret().isAnythingSelected())
    {
        deleteSelection();
    }
    else if(caret().dot() > 0)
    {
        std::string s1 = stringUpToCaret();
        s1 = s1.substr(0, s1.length()-1);
        setText(s1 + stringFromCaret());
        caret().setDot(caret().dot()-1);
    }
    caret().removeSelection();


    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
EditWindow&
EditWindow::doDelete()
{
    if(caret().isAnythingSelected())
    {
        deleteSelection();
    }
    else if(caret().dot() < text().length())
    {
        std::string s = stringFromCaret();
        s = s.substr(1, s.length()-1);
        setText(stringUpToCaret() + s);
    }
    caret().removeSelection();

    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
EditWindow&
EditWindow::deleteSelection()
{
    std::string s = text();
    s = s.substr(0, caret().selectionStart()) + 
        s.substr(caret().selectionEnd(), s.length() - caret().selectionEnd());
    caret().setDot(caret().selectionStart());
    caret().removeSelection();
    setText(s);
    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<








// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
std::string
EditWindow::stringUpToCaret() const
{
    std::string ret;
    if(caret().dot() > 0) ret = text().substr(0, caret().dot());

    return ret;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
std::string
EditWindow::stringFromCaret() const
{
    std::string ret;
    if(caret().dot() < text().length())
        ret = text().substr(caret().dot(), text().length());

    return ret;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<









// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
bool
EditWindow::onKeyPressed(Key::Code aKeyCode)
{
    bool resetIdealCaretXPos = true;
    bool isHandled = false;
    if(aKeyCode == Key::left)
    {
        size_t index = (caret().dot() > 0) ? caret().dot()-1 : 0;
        if(WindowManager::instance().isCtrlPressed())
            index = lastSpaceBefore(caret().dot());
        setCaretPositionConsiderShiftKey(index);
        isHandled = true;
    }
    else if(aKeyCode == Key::right)
    {
        size_t index = (caret().dot() < text().length()) ? caret().dot()+1 : text().length();
        if(WindowManager::instance().isCtrlPressed())
            index = nextWordStartPosAfter(caret().dot());
        setCaretPositionConsiderShiftKey(index);
        isHandled = true;
    }
    else if(aKeyCode == Key::up)
    {
        if(isMultiline() && (caretLine() > 0))
        {
            if(idealCaretXPos_ < 0.0f) idealCaretXPos_ = caretXPos()+1.0f;
            Core::Vector2 pos(idealCaretXPos_, renderDesc().lineCenter(caretLine()-1));
            setCaretPositionConsiderShiftKey(hitCharacter(pos));
            resetIdealCaretXPos = false;
        }
        isHandled = true;
    }
    else if(aKeyCode == Key::down)
    {
        if(isMultiline() && (caretLine() < numberOfLines()-1))
        {
            if(idealCaretXPos_ < 0.0f) idealCaretXPos_ = caretXPos()+1.0f;
            Core::Vector2 pos(idealCaretXPos_, renderDesc().lineCenter(caretLine()+1));
            setCaretPositionConsiderShiftKey(hitCharacter(pos));
            resetIdealCaretXPos = false;
        }
        isHandled = true;
    }
    else if(aKeyCode == Key::home)
    {
        if(WindowManager::instance().isCtrlPressed())
            setCaretPositionConsiderShiftKey(0);
        else
            setCaretPositionConsiderShiftKey(lineStart(caretLine()));
        isHandled = true;
    }
    else if(aKeyCode == Key::end)
    {
        if(WindowManager::instance().isCtrlPressed())
            setCaretPositionConsiderShiftKey(text().length());
        else
            setCaretPositionConsiderShiftKey(lineEnd(caretLine()));
        isHandled = true;
    }
    else if(aKeyCode == Key::del)
    {
        doDelete();
    }
    else if(aKeyCode < Key::del)
    {
        onCharacterInput(aKeyCode);
    }
    else if( (aKeyCode >= Key::world0) && (aKeyCode <= Key::world95) )
    {
        insertCharacter(static_cast<char>(aKeyCode));
    }

    if(resetIdealCaretXPos) idealCaretXPos_ = -1.0f;

    
    return isHandled;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
bool
EditWindow::onCharacterInput(Key::Code aKeyCode)
{
    idealCaretXPos_ = -1.0f;
    
    const unsigned char aCharacter = static_cast<unsigned char>(aKeyCode);
    
    if(WindowManager::instance().isCtrlPressed())
    {
        switch(aCharacter)
        {
        case Key::a:
            caret().select(0, text().length());
            break;
        case Key::c:
            if(selectedText() != "")
                WindowManager::instance().copyToClipboard(selectedText());
            break;
        case Key::v:
            {
                std::string str = text();
                std::string pasteText = WindowManager::instance().clipboardContents();
                str.insert(caret().dot(), pasteText);
                setText(str);
                caret().setDot(caret().dot() + pasteText.length());
            }
            break;
        case Key::x:
            if(caret().isAnythingSelected())
            {
                WindowManager::instance().copyToClipboard(selectedText());
                std::string str = text();
                str.erase(caret().selectionStart(), caret().selectionSize());
                caret().setDot(caret().selectionStart());
                setText(str);
            }
            break;
        default:
            break;
        }
    }
    else
    {
        switch(aCharacter)
        {
        case Key::backspace:
            doBackspace();
            break;
        case Key::ret:
            returnPressed();
            break;
        default:
            if(aCharacter > 31)
            {
                if( (aCharacter > 96) && (aCharacter <= 122) && WindowManager::instance().isShiftPressed() )
                    insertCharacter(aCharacter - 32);
                else
                    insertCharacter(aCharacter);
            }
            break;
        }
    }
    return true;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<








// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void
EditWindow::onTextChanged()
{
    lineEndPositions_.clear();
    size_t pos = 0;
    while(pos != std::string::npos)
    {
        pos = text().find_first_of("\n", pos);
        lineEndPositions_.push_back( (pos != std::string::npos) ? pos : text().length() );
        if(pos != std::string::npos) ++pos;
    }
    textChanged(text());
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<








// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
EditWindow& 
EditWindow::insertCharacter(char aCharacter)
{
    if(caret().isAnythingSelected()) deleteSelection();
    setText(stringUpToCaret() + Core::toString(aCharacter) + stringFromCaret());
    caret().setDot(caret().dot() + 1);
    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<








// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
EditWindow& 
EditWindow::setCaretPositionConsiderShiftKey(size_t anIndex)
{
    if(WindowManager::instance().isShiftPressed())
        caret().moveDot(anIndex);
    else
        caret().setDot(anIndex);
    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
size_t
EditWindow::lastSpaceBefore(size_t pos) const
{
    // get the string up to pos
    std::string str = text().substr(0, pos);
    size_t i = 0;
    while(i != std::string::npos)
    {
        i = str.find_first_of('\n');
        if(i != std::string::npos) str[i] = ' ';
    }

    size_t ret = 0;
    if(str.length() > 0)
    {
        // strip any trailing spaces
        std::string::size_type n = str.find_last_not_of(' ');
        if(n != std::string::npos) str.erase(n, str.length()-n);

        // return the position of the last space
        const size_t p = str.find_last_of(' ');

        // if we couldn't find a space we return the beginning of the string
        if(p != std::string::npos) ret = p + 1;
    }
    return ret;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
size_t
EditWindow::nextWordStartPosAfter(size_t pos) const
{
    // get the string after pos
    std::string str = text().substr(pos, text().length()-pos);
    size_t i = 0;
    while(i != std::string::npos)
    {
        i = str.find_first_of('\n');
        if(i != std::string::npos) str[i] = ' ';
    }

    // find out how many leading spaces there are
    const size_t firstNonSpacePos = str.find_first_not_of(' ');

    size_t ret = text().length();
    if(firstNonSpacePos != std::string::npos)
    {
        // strip leading spaces
        str.erase(0, firstNonSpacePos);

        // now find the next space
        const size_t p = str.find_first_of(' ');

        if(p != std::string::npos)
        {
            // pass over any leading spaces
            std::string str2 = str.substr(p, str.length());
            const size_t p2 = str2.find_first_not_of(' ');
            if(p2 != std::string::npos)
                ret = p2 + p + pos;
        }
    }

    return ret;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
std::string
EditWindow::selectedText() const
{
    std::string ret;
    if(caret().isAnythingSelected()) 
        ret = text().substr(caret().selectionStart(), caret().selectionSize());

    return ret;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
EditWindowDesc&
EditWindow::renderDesc() const
{
    EditWindowDesc& result = (renderDesc_ != 0) ? *renderDesc_ : WindowManager::instance().propertyScheme().editWindowDesc_;
    result.setEditWindow(this);
    return result;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
EditWindow&
EditWindow::onCaretMoved(size_t pos)
{
    //  we'll need to adjust the visible part of the text if the caret has
    // gone off one of the ends
    const float xPos = caretXPos();
    const float leftSideX = position().x() + renderDesc().textDesc().margin();
    const float rightSideX = leftSideX + size().x() - (renderDesc().textDesc().margin() * 2.0f);
    if(xPos < leftSideX)
    {
        xOffset_ += (leftSideX - xPos);
    }
    else if(xPos > rightSideX)
    {
        xOffset_ -= (xPos - rightSideX);
    }

    return *this; 
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<









// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
EditWindow&
EditWindow::selectAll()
{
    caret().select(0, text().length());
    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
float
EditWindow::caretXPos() const
{
    TextDesc& textDesc = renderDesc().textDesc();
        
    std::string lineText = stringUpToCaret();
    const size_t startIndex = lineStart(caretLine());
    float textX = 0.0f;
    if(lineText.length() >= startIndex)
    {
        lineText = lineText.substr(startIndex, lineText.length() - startIndex);
        textDesc.setText(lineText);
        textX = textDesc.textSize().x();
    }
    return textX + xOffset_ + windowRectangle().bottomLeft().x() + textDesc.margin();
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<








// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
Core::Vector2
EditWindow::recommendedSize() const
{
    EditWindowDesc* desc = &renderDesc();
    if(desc == 0) desc = &WindowManager::instance().propertyScheme().editWindowDesc_;

    desc->textDesc().setText(text());
    return desc->textDesc().textSize() + Core::Vector2(8, 8);
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
size_t 
EditWindow::lineStart(size_t aLineIndex) const
{
    size_t result = std::string::npos;
    if(aLineIndex < numberOfLines()) result = (aLineIndex == 0) ? 0 : lineEndPositions_[aLineIndex-1]+1;
    return result;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
size_t 
EditWindow::lineEnd(size_t aLineIndex) const
{
    size_t result = std::string::npos;
    if(aLineIndex < numberOfLines()) result = lineEndPositions_[aLineIndex];
    return result;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
size_t 
EditWindow::caretLine() const
{
    size_t result = numberOfLines() > 0 ? numberOfLines()-1 : 0;
    for(size_t i = 0; i < numberOfLines(); ++i)
    {
        if( (caret().dot() >= lineStart(i)) && (caret().dot() <= lineEnd(i)) ) result = i;
    }
    return result;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
Window&
EditWindow::draw()
{
    renderDesc()
        .setEditWindow(this)
        .draw(drawRectangle())
        ;

    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
Core::Rectangle EditWindow::drawRectangle() const
{
    const Core::Vector2 topLeft = position();
    const Core::Vector2 bottomRight = position() + size();
    return Core::Rectangle(topLeft, bottomRight);
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
size_t 
EditWindow::hitCharacter(const Core::Vector2& aPosition) const
{
    using namespace Core;
    const Vector2 localPosition = aPosition - position() - 
        Vector2(renderDesc().textDesc().margin() + xOffset_, renderDesc().textDesc().margin());
                
    size_t result = 0;
    if(isMultiline())
    {
        const float lineHeight = renderDesc().textDesc().lineHeight();
        if(lineHeight > 0.0f)
        {
            const int line = int(firstVisibleLine()) + int(floor(localPosition.y() / lineHeight));
            if(line >= int(numberOfLines()))
                result = text().length();
            else if(line >= 0)
            {
                const size_t start = lineStart(line);
                const size_t end   = lineEnd(line);
                const std::string lineText = text().substr(start, end - start);
                const int index = renderDesc().textDesc().hitCharacterIndex(lineText, localPosition.x());
                result = (index >= 0) ? static_cast<size_t>(index) + start : end;
            }
        }        
    }
    else
    {
        const int index = renderDesc().textDesc().hitCharacterIndex(text(), localPosition.x());
        result = (index < 0) ? text().length() : static_cast<size_t>(index);
    }
    return result;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
Window&
EditWindow::loadAppearance(Core::Archive& anArchive, const std::string& aWindowName)
{
    anArchive & Core::nameObject(renderDesc_, aWindowName.c_str());
    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void
EditWindow::serialize(Core::Archive& anArchive)
{        
    anArchive & TAGGED_OBJECT(renderDesc);
    Window::serialize(anArchive);
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<










// **************************************************************************************************
//                                            EditWindowDesc
// **************************************************************************************************




// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
EditWindowDesc::EditWindowDesc() :
    background_(new SolidSurfaceDesc()),
    border_(new RectangleDesc()),
    textDesc_(new TextDesc()) ,
    selectionColor_(Color(0.85f, 0.85f, 0.85f, 1)),
    selectedTextColor_(Color(0.35f, 0.35f, 0.35f, 1)),
    editWindow_(0)
{
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
EditWindowDesc::~EditWindowDesc()
{
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void
EditWindowDesc::draw(const Core::Rectangle& anArea) const
{
    textDesc_->setHorizontalAlignment(Core::Alignment::left);
    if(editWindow_->isMultiline()) textDesc_->setVerticalAlignment(Core::Alignment::top);

    if(background_ != 0) background_->draw(anArea);

    if(editWindow_->text() != "")
    {
        Core::Rectangle area = anArea;

        Core::Rectangle clipRectangle(area);
        clipRectangle.expandBy(-textDesc_->margin());
        Renderer::enableClipRectangle(clipRectangle);

        drawTextWithSelection(area);
    }
    
    drawCaret();
    
    Renderer::disableClipRectangle();

    if(border_ != 0) border_->draw(anArea);

    Renderer::setCurrentColor(Color(1, 1, 1, 1));
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void 
EditWindowDesc::drawLine(const Core::Rectangle& anArea, size_t aLineIndex) const
{
    const size_t lineStart = editWindow_->lineStart(aLineIndex);
    const size_t lineEnd = editWindow_->lineEnd(aLineIndex);
    ASSERT(lineStart != std::string::npos);
    ASSERT(lineEnd != std::string::npos);
    ASSERT(lineEnd >= lineStart);
    const size_t lineSize = lineEnd - lineStart;
    const std::string lineText = editWindow_->text().substr(lineStart, lineSize);
    
    size_t selectionStart = editWindow_->caret().selectionStart();
    size_t selectionEnd   = editWindow_->caret().selectionEnd();
    if( editWindow_->caret().isAnythingSelected() && (selectionStart < lineEnd) && (selectionEnd >= lineStart) )
    {
        selectionStart = (selectionStart > lineStart) ? selectionStart - lineStart : 0;
        selectionEnd   = (selectionEnd < lineEnd) ? selectionEnd - lineStart : lineText.length();
    
        // draw pre selected text
        Core::Rectangle textRectangle(anArea);
        textRectangle.translateBy(Core::Vector2(editWindow_->xOffset_, 0.0f));
        const float preSelectedWidth = drawPreSelectedText(lineText, textRectangle, selectionStart);
    
        // draw selected text
        textRectangle.translateBy(Core::Vector2(preSelectedWidth, 0.0f));
        const float selectedWidth = drawSelectedText(lineText, textRectangle, selectionStart, 
                                                     selectionEnd-selectionStart, anArea, preSelectedWidth);
    
        // Finally do any text after the selection
        textRectangle.translateBy(Core::Vector2(selectedWidth, 0.0f));
        drawPostSelectedText(lineText, textRectangle, selectionEnd);
    }
    else
    {
        drawTextWithoutSelection(lineText, anArea);
    }
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void
EditWindowDesc::drawTextWithSelection(const Core::Rectangle& anArea) const
{
    Core::Rectangle area(anArea);

    // We draw in 3 steps and don't want the textDesc's margin to appear each time
    const float margin = textDesc_->margin();
    textDesc_->setMargin(0.0f);
    area.translateBy(Core::Vector2(0.0f, margin));
    area.expandWidthBy(-margin);

    const size_t firstLineToDraw = editWindow_->firstVisibleLine();
    for(size_t lineIndex = firstLineToDraw; lineIndex < editWindow_->numberOfLines(); ++lineIndex)
    {
        drawLine(area, lineIndex);
        area.translateBy(Core::Vector2(0.0f, textDesc_->lineHeight()));
    }

    // Set the margin back
    textDesc_->setMargin(margin);
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void
EditWindowDesc::drawTextWithoutSelection(const std::string& aText, const Core::Rectangle& anArea) const
{
    Core::Rectangle textRectangle(anArea);
    textRectangle.translateBy(Core::Vector2(editWindow_->xOffset_, 0.0f));
    (*textDesc_)
        .setText(aText)
        .draw(textRectangle)
        ;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
float
EditWindowDesc::drawPreSelectedText(const std::string& aText, const Core::Rectangle& anArea, size_t aSelectionStart) const
{
    const std::string text = aText.substr(0, aSelectionStart);
    textDesc_->setText(text);

    (*textDesc_).draw(anArea);

    return textDescTextWidth();
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
float
EditWindowDesc::drawSelectedText(const std::string& aText, const Core::Rectangle& anArea, 
    size_t aSelectionStart, size_t aSelectionSize, const Core::Rectangle& aVisibleArea, float aPreSelectedWidth) const
{
    const std::string selected = aText.substr(aSelectionStart, aSelectionSize);
    textDesc_->setText(selected);

    drawSelectionBackground(aVisibleArea, editWindow_->xOffset_ + aPreSelectedWidth);

    const Color unselectedColor = textDesc_->color();
    (*textDesc_)
        .setColor(selectedTextColor_)
        .draw(anArea)
        ;

    textDesc_->setColor(unselectedColor);

    return textDescTextWidth();
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void
EditWindowDesc::drawPostSelectedText(const std::string& aText, const Core::Rectangle& anArea, size_t aSelectionEnd) const
{
    std::string postSelected = aText.substr(aSelectionEnd, aText.length() - aSelectionEnd);

    (*textDesc_)
        .setText(postSelected)
        .draw(anArea)
        ;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void
EditWindowDesc::drawSelectionBackground(const Core::Rectangle& aVisibleArea, float aStartX) const
{
    Renderer::setCurrentColor(selectionColor_);
    if(editWindow_->isMultiline())
    {
        const Core::Vector2 topLeft = aVisibleArea.topLeft() + Core::Vector2(aStartX, 1.0f);
        const Core::Vector2 bottomRight(topLeft.x()+textDescTextWidth(), topLeft.y() + textDesc_->lineHeight());
        Renderer::drawRectangle(topLeft, bottomRight, true);
    }
    else
    {
        const Core::Vector2 topLeft = aVisibleArea.topLeft() + Core::Vector2(aStartX, 1.0f);
        const Core::Vector2 bottomRight(topLeft.x()+textDescTextWidth(), aVisibleArea.bottomRight().y()-1.0f);
        Renderer::drawRectangle(topLeft, bottomRight, true);
    }
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void
EditWindowDesc::drawCaret() const
{
    if(editWindow_->isFocused())
    {
        if(static_cast<int>(TimeManager::systemTime()*2.0f) % 2)
        {
            Renderer::setCurrentColor(selectionColor_);
            
            const float x = editWindow_->caretXPos();
            const float caretHeightOverTwo = (textDesc_->lineHeight()) * 0.5f;
            const float verticalCenter = lineCenter(editWindow_->caretLine());
    
            Renderer::drawLine(Core::Vector2(x, verticalCenter - caretHeightOverTwo),
                    Core::Vector2(x, verticalCenter + caretHeightOverTwo));
        }
    }
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
float
EditWindowDesc::textDescTextWidth() const
{
    float width = textDesc_->textSize().x();
    if(width < 0.0f) width = 0.0f;
    return width;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
float 
EditWindowDesc::lineCenter(size_t aLineIndex) const
{
    float result = 0;
    if(editWindow_->isMultiline())
    {
        result = editWindow_->windowRectangle().topLeft().y() +
            textDesc_->margin() + 
            (static_cast<float>(int(aLineIndex) - int(editWindow_->firstVisibleLine())) * textDesc_->lineHeight()) +
            (textDesc_->lineHeight() * 0.5f);
    }
    else
    {
        result = (editWindow_->windowRectangle().size().y() * 0.5f) + 
                  editWindow_->windowRectangle().topLeft().y();
    }
    return result;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void
EditWindowDesc::serialize(Core::Archive& anArchive)
{
    RenderDesc::serialize(anArchive);
    anArchive &
        TAGGED_OBJECT(background) &
        TAGGED_OBJECT(border) &
        TAGGED_OBJECT(textDesc) &
        TAGGED_OBJECT(selectionColor) &
        TAGGED_OBJECT(selectedTextColor);
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<




// *************************************************************************************************
} // namespace Gooey
