// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________
#ifndef __GOOEY_EDIT_WINDOW_H__
#define __GOOEY_EDIT_WINDOW_H__


#include "glgooey/Window.h"
#include "glgooey/RenderDesc.h"
#include "glgooey/KeyMessage.h"
#include "glgooey/Caret.h"


// *************************************************************************************************
//  Forward declarations
namespace Gooey
{
    class EditWindowMouseListener;
    DEFINE_SMARTPOINTER(EditWindowDesc);
    DEFINE_SMARTPOINTER(TextDesc);
}
// *************************************************************************************************


// *************************************************************************************************
namespace Gooey
{


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
class EditWindow : public Window, public KeyListener
{
    // ---------------------------------------------------------------------------------------------
    //  Construction / Destruction
    // ---------------------------------------------------------------------------------------------
public:
    //! \brief constructor requires the parent, the text with which the edit window
    //! is to be initialized and whether multiline editing is supported
    EditWindow(Window* aParent, const std::string& aText, bool multiline);

    //! destructor
    virtual ~EditWindow();


    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    //! Returns the currently selected text (empty if nothing is selected)
    std::string selectedText() const;
    
    //! returns a constant reference to the caret
    const Caret& caret() const { return caret_; }

    //! Returns the number of lines of text in the edit window
    size_t numberOfLines() const { return lineEndPositions_.size(); }
    
    //! Position of the starting character of the passed in line
    size_t lineStart(size_t aLineIndex) const;
    
    //! Returns the number of lines of text in the edit window
    size_t lineEnd(size_t aLineIndex) const;

    //! Returns true if this is an edit window which supports multiple lines
    bool isMultiline() const { return isMultiline_; }
        
    //! Commented in base class
    virtual bool onCharacterInput(Key::Code aKeyCode);
    virtual bool onKeyPressed(Key::Code aKeyCode);    

    //! \brief Sets the current selection to contain the entire contents of the
    //! edit field
    EditWindow& selectAll();

    // Commented in base class
    virtual Core::Vector2 recommendedSize() const;

    // Commented in base class
    virtual Window& loadAppearance(Core::Archive& anArchive, const std::string& aWindowName);



    // ---------------------------------------------------------------------------------------------
    //  Signals
    // ---------------------------------------------------------------------------------------------
public:
    //! \brief This signal is emitted if return is pressed while the edit field
    //! has the focus
    Signal0 returnPressed;

    //!  This signal is emitted when the text in the edit field changes
    Signal1<const std::string&> textChanged;

        
    // ---------------------------------------------------------------------------------------------
    //  Implementation
    // ---------------------------------------------------------------------------------------------
protected:
    //! returns a reference to the caret
    Caret& caret() { return caret_; }
    
    //! react to the user entering a backspace
    EditWindow& doBackspace();

    //! react to the user pressing the delete key
    EditWindow& doDelete();

    //! deletes the currently selected text
    EditWindow& deleteSelection();

    //! returns the sub string up to the caret
    std::string stringUpToCaret() const;

    //! returns the sub string from the caret to the end
    std::string stringFromCaret() const;

    //! Returns a pointer to the render desc that will be used to draw this edit window
    EditWindowDesc& renderDesc() const;
    
    //! returns the xPos of the caret in app window coords
    float caretXPos() const;
    
    //! render the window to the current OpenGL context
    virtual Window& draw();

    //! derived classes can override this to return an alternative rectangle upon which to draw
    virtual Core::Rectangle drawRectangle() const;
    
    //! React to the text changing
    virtual void onTextChanged();
    
    //! Returns the zero based index of the first visible line of text
    virtual size_t firstVisibleLine() const { return 0; }
    
    //! \brief Reacts to the caret moving by changing the visible text if necessary, ensuring that
    //! the caret remains visible
    virtual EditWindow& onCaretMoved(size_t pos);
    
    //! Returns the index of the line with the caret in it
    size_t caretLine() const;
    
    //! inserts a character into the string at the current caret position
    EditWindow& insertCharacter(char aCharacter);
    
private:
    //! Sets the position of the caret without losing the selection if the shift key is pressed
    EditWindow& setCaretPositionConsiderShiftKey(size_t anIndex);

    //! returns the position of the last space character before the passed in position
    size_t lastSpaceBefore(size_t pos) const;

    //! returns the position of the beginning of the next word after the passed in position
    size_t nextWordStartPosAfter(size_t pos) const;
    
    //! Returns the nearest character to the passed in position
    size_t hitCharacter(const Core::Vector2& aPosition) const;
    
    Caret caret_;  //!< The caret
    
    EditWindowDescPtr renderDesc_;
    friend class EditWindowDesc;

    friend class EditWindowMouseListener;
    EditWindowMouseListener* mouseListener_;

    float xOffset_;
    
    std::vector<size_t> lineEndPositions_;
    
    bool isMultiline_;
    
    float idealCaretXPos_;  //!< Where the caret would ideally be if the line was long enough
    
           
    // ---------------------------------------------------------------------------------------------
    //  Serialization
    // ----------------------------------------------------------------------------------------------
    DECLARE_SERIALIZATION;    
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<




// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! \brief The render desc class for edit windows
class EditWindowDesc : public RenderDesc
{
    // ---------------------------------------------------------------------------------------------
    //  Construction / Destruction
    // ---------------------------------------------------------------------------------------------
public:
    //! Default Constructor
    EditWindowDesc();

    //! Destructor
    virtual ~EditWindowDesc();


    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    //! Draws on the passed in area according to the desc
    virtual void draw(const Core::Rectangle& area) const;

    //! Sets the edit field to be rendered on the next call to draw
    EditWindowDesc& setEditWindow(const EditWindow* anEditWindow)
    {
        editWindow_ = anEditWindow;
        return *this;
    }
    
    //! Returns a reference to the text desc
    TextDesc& textDesc() { return *textDesc_; }

    //! Returns the y coordinate of the center of the passed in line
    float lineCenter(size_t aLineIndex) const;

    
    // ---------------------------------------------------------------------------------------------
    //  Implementation
    // ---------------------------------------------------------------------------------------------
private:
    //! Draws the text when some of it is selected
    void drawTextWithSelection(const Core::Rectangle& anArea) const;

    //! Draws the text when no text is selected
    void drawTextWithoutSelection(const std::string& aText, const Core::Rectangle& anArea) const;

    //! \brief Draws the text up to the start of the selection on the passed in area
    //! \returns the width of the text that was drawn
    float drawPreSelectedText(const std::string& aText, const Core::Rectangle& anArea, size_t aSelectionStart) const;

    //! \brief Draws the selection mask and the selected text on the passed in area. The width
    //! of the text up until the start of the selection must also be passed in.
    //! \returns the width of the text that was drawn
    float drawSelectedText(const std::string& aText, const Core::Rectangle& anArea, 
        size_t aSelectionStart, size_t aSelectionSize, const Core::Rectangle& aVisibleArea, float aPreSelectedWidth) const;

    //! \brief Draws the text after the end of the selection on the passed in area
    void drawPostSelectedText(const std::string& aText, const Core::Rectangle& anArea, size_t aSelectionEnd) const;

    //! returns the width of the text currently set in the text desc
    float textDescTextWidth() const;

    //! draws the selection background
    void drawSelectionBackground(const Core::Rectangle& aVisibleArea, float aStartX) const;
    
    //! draws the line of text defined by the passed in line index
    void drawLine(const Core::Rectangle& anArea, size_t aLineIndex) const;

    //! draws the caret
    void drawCaret() const;

    RenderDescPtr  background_;
    RenderDescPtr  border_;
    TextDescPtr    textDesc_;
    Color        selectionColor_;
    Color        selectedTextColor_;
    const EditWindow*  editWindow_;


    // ---------------------------------------------------------------------------------------------
    //  Serialization
    // ----------------------------------------------------------------------------------------------
    DECLARE_SERIALIZATION;
    
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<



// *************************************************************************************************
} // namespace

#endif
