// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________

#include "glgooey/FrameWindow.h"

#include "glgooey/Renderer.h"
#include "glgooey/WindowManager.h"
#include "glgooey/PropertyScheme.h"
#include "glgooey/core/Rectangle.h"
#include "glgooey/core/StandardException.h"
#include "glgooey/MouseMessage.h"
#include "glgooey/SolidSurfaceDesc.h"


// *************************************************************************************************
namespace Gooey
{

REGISTER_CLASS(FrameWindowDesc);


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
class FrameWindowMouseListener : public MouseListener
{
    // ---------------------------------------------------------------------------------------------
    //  Data Types
    // ---------------------------------------------------------------------------------------------
public:
    enum ResizeModeFlags { disabled = 0x00, left = 0x01, right = 0x02, bottom = 0x04, top = 0x08, 
                           horizontal = left | right, vertical = bottom | top,
                           topleft = top | left, topright = top | right, bottomleft = bottom | left, bottomright = bottom | right };
    
    typedef unsigned int ResizeMode;    

private:
    //! the possible handle modes of the frame window
    enum HandleMode { none, move, resize }; 
    
    
    
    // ---------------------------------------------------------------------------------------------
    //  Construction / Destruction
    // ---------------------------------------------------------------------------------------------
public:
    //! construction requires a reference to the associated frame window
    FrameWindowMouseListener(FrameWindow& aFrameWindow) : frameWindow_(aFrameWindow) {}

    //! destructor
    virtual ~FrameWindowMouseListener() {}


    // ---------------------------------------------------------------------------------------------
    //  Implementation
    // ---------------------------------------------------------------------------------------------
protected:
    //! process the moved message by moving the window if it is currently pressed
    virtual bool moved(const MouseMessage& message);

    //! process the click message by moving the window if it is currently pressed
    virtual bool clicked(const MouseClickMessage& message);

private:
    void setHandleMode(const Core::Vector2& position);

private:    
    FrameWindow& frameWindow_;
    Core::Vector2 oldMousePos_;
    HandleMode handleMode_;
    ResizeMode resizeMode_;

    // Copying and assignment are not allowed
    FrameWindowMouseListener(const FrameWindowMouseListener& aFrameWindowMouseListener);
    FrameWindowMouseListener& operator = (const FrameWindowMouseListener& aFrameWindowMouseListener);
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// **************************************************************************************************
//                                            FrameWindow
// **************************************************************************************************




// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
FrameWindow::FrameWindow(const Core::Rectangle& rect, Window* aParent,
                         const std::string& aText) :
    Window(aParent, aText),
    clientWindow_(0),
    captureCount_(0),
    isMovable_(false),
    isResizable_(false),
    renderDesc_(0)
{
    // set the geometry
    setFrameRectangle(rect);

    mouseListener_ = new FrameWindowMouseListener(*this);
    addMessageListener(mouseListener_);
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
FrameWindow::~FrameWindow()
{
    removeMessageListener(mouseListener_);
    delete mouseListener_;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
Core::Rectangle
FrameWindow::frameRectangle() const
{
    FrameWindowDesc* renderDesc = renderDesc_;
    if(renderDesc == 0)
        renderDesc = &WindowManager::instance().propertyScheme().frameWindowDesc_;

    const Core::Vector2 topLeft = position() -
        Core::Vector2(renderDesc->leftBorderWidth(), renderDesc->topBorderWidth());
    const Core::Vector2 bottomRight = position() + size() +
        Core::Vector2(renderDesc->rightBorderWidth(), renderDesc->bottomBorderWidth());
    return Core::Rectangle(topLeft, bottomRight);    
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
Core::Rectangle
FrameWindow::clientRectangle() const
{
    return windowRectangle();
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
FrameWindow&
FrameWindow::setClientRectangle(const Core::Rectangle& rect)
{
    setWindowRectangle(rect);
    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
FrameWindow&
FrameWindow::setFrameRectangle(const Core::Rectangle& rect)
{
    FrameWindowDesc* renderDesc = renderDesc_;
    if(renderDesc == 0)
        renderDesc = &WindowManager::instance().propertyScheme().frameWindowDesc_;

    const Core::Vector2 topLeft = rect.topLeft() +
        Core::Vector2(renderDesc->leftBorderWidth(), renderDesc->topBorderWidth());
    const Core::Vector2 bottomRight = rect.bottomRight() -
        Core::Vector2(renderDesc->rightBorderWidth(), renderDesc->bottomBorderWidth());
    
    setWindowRectangle(Core::Rectangle(topLeft, bottomRight));
    
    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
FrameWindow&
FrameWindow::clientWindowChanged()
{
    if(clientWindow() != 0)
        clientWindow()->setWindowRectangle(clientRectangle());

    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
bool
FrameWindow::process(const Message& message)
{
    bool ret = false;
    if(isVisible() && isEnabled())
    {
        if(clientWindow() != 0)
            ret = clientWindow()->process(message);
        if(!ret) ret = Window::process(message);
    }

    return ret;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
bool
FrameWindow::doesHierarchyContain(Window* aWindow) const
{
    bool ret = false;
    if(clientWindow() != 0)
        ret = clientWindow()->doesHierarchyContain(aWindow);
    return (ret || Window::doesHierarchyContain(aWindow));
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
Core::Vector2
FrameWindow::recommendedSize() const
{
    return Core::Vector2(200, 200);
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
Window&
FrameWindow::draw()
{
    FrameWindowDesc* renderDesc = renderDesc_;
    if(renderDesc == 0)
        renderDesc = &WindowManager::instance().propertyScheme().frameWindowDesc_;

    renderDesc->setFrameWindow(this);

    renderDesc->draw(frameRectangle());

    if(clientWindow() != 0)
    {
        Renderer::enableClipRectangle(clientRectangle());
        clientWindow()->render();
        Renderer::disableClipRectangle();
    }


    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
bool
FrameWindow::isBelow(const Core::Vector2& aPosition)
{
    return frameRectangle().surrounds(aPosition);
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
Window*
FrameWindow::windowBelow(const Core::Vector2& p)
{
    Window* ret = 0;
    if(!isVisible() || !frameRectangle().surrounds(p))
    {
        return ret;
    }
    ret = this;

    if(clientWindow())
    {
        Window* cw = clientWindow()->windowBelow(p);
        if(cw != 0) ret = cw;
    }
    return ret;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
bool
FrameWindow::isMovable() const
{
    return isMovable_;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
bool
FrameWindow::isResizable() const
{
    return isResizable_;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
FrameWindow&
FrameWindow::enableMovement()
{
    isMovable_ = true;
    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
FrameWindow&
FrameWindow::disableMovement()
{
    isMovable_ = false;
    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
FrameWindow&
FrameWindow::enableResize()
{
    isResizable_ = true;
    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
FrameWindow&
FrameWindow::disableResize()
{
    isResizable_ = false;
    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void
FrameWindow::onResizing(Core::Vector2& size)
{    
    if(size.x() < 0)
        size.setX(0);
    
    if(size.y() < 0)
        size.setY(0);

    if(clientWindow() != 0)
        clientWindow()->setWindowRectangle(Core::Rectangle(position(), position() + size));
} 
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void
FrameWindow::onResize()
{    
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void
FrameWindow::onMove()
{
    if(clientWindow() != 0)
        clientWindow()->setPosition(clientRectangle().position());
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
Window&
FrameWindow::loadAppearance(Core::Archive& anArchive, const std::string& aWindowName)
{
    anArchive & Core::nameObject(renderDesc_, aWindowName.c_str());
    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// **************************************************************************************************
//                                        FrameWindowMouseListener
// **************************************************************************************************




// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
bool
FrameWindowMouseListener::moved(const MouseMessage& message)
{
    bool isHandled = false;
    if(handleMode_ != none)
    {
        if(WindowManager::instance().captureWindow() == &frameWindow_)
        {
            Core::Vector2 delta(message.x() - oldMousePos_.x(), message.y() - oldMousePos_.y());
            Core::Vector2 adjustDelta(1,1);
            
            if(handleMode_ == move)
            {
                frameWindow_.setPosition(frameWindow_.position() + delta);
                oldMousePos_ = message.position();
            }
            else if(handleMode_ == resize)
            {                   
                Core::Vector2 oldSize;
                
                if(resizeMode_ & horizontal)
                {                
                    if(resizeMode_ & left)
                    {
                        // if we drag on the left side we have to
                        // move the window along the x-axis to lock the 
                        // right side
                        delta.setX(-delta.x());
                        adjustDelta.setX(-1);
                    }
                }
                else
                {
                    delta.setX(0);
                }
                
                if(resizeMode_ & vertical)
                {
                    if(resizeMode_ & top)
                    {
                        // if we drag on the top we have to
                        // move the window along the y-axis to lock the 
                        // bottom side
                        adjustDelta.setY(-1);
                        delta.setY(-delta.y());
                    }
                }
                else
                {
                    delta.setY(0);
                }
                
                // remember the old size
                oldSize = frameWindow_.size();
                
                // set the requestes size
                frameWindow_.setSize(frameWindow_.size() + delta);
                
                // read back the current size and calculate the "real"
                // distance between the old and the new size
                delta = frameWindow_.size() - oldSize;
                
                delta *= adjustDelta;
                oldMousePos_ += delta;
                
                // disable unwanted directions
                if(adjustDelta.x() > 0)
                    delta.setX(0);
                
                if(adjustDelta.y() > 0)
                    delta.setY(0);
  
                // finally set the new position
                frameWindow_.setPosition(frameWindow_.position() + delta);
            }
        }        
        isHandled = true;
    }

    return isHandled;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
bool
FrameWindowMouseListener::clicked(const MouseClickMessage& message)
{
    bool isHandled = false;        
            
    if(message.button() == MouseClickMessage::left)
    {
        if(message.event() == MouseClickMessage::pressed)
        {            
            setHandleMode(message.position());
            
            if(handleMode_ != none)
            {
                if(WindowManager::instance().captureWindow() != &frameWindow_)
                    WindowManager::instance().setMouseCapture(&frameWindow_);
                frameWindow_.captureCount_++;
            }
            oldMousePos_ = message.position();
            frameWindow_.setAsActiveWindow();
            isHandled = true;
        }
        else if(message.event() == MouseClickMessage::released)
        {
            if(frameWindow_.isMovable())
            {
                if(frameWindow_.captureCount_ == 1)
                    WindowManager::instance().releaseMouseCapture();
                if(frameWindow_.captureCount_ > 0)
                    frameWindow_.captureCount_--;
            }
            isHandled = true;
        }
    }
    return isHandled;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void
FrameWindowMouseListener::setHandleMode(const Core::Vector2& position)
{
    if(frameWindow_.isResizable())
    {
        Core::Rectangle frameRect = frameWindow_.frameRectangle();
        Core::Rectangle moveRect(frameRect);
        
        moveRect.expandBy(-10);
        
        if(moveRect.surrounds(position))
        {
            if(frameWindow_.isMovable())
                handleMode_ = move;
        }
        else
        {
            if(frameWindow_.isResizable())
            {
                handleMode_ = resize;
                resizeMode_ = disabled;
                
                // get the current resize mode
                if(position.y()<moveRect.position().y())
                    resizeMode_ = top;
                else if(position.y()>moveRect.position().y() + moveRect.size().y())
                    resizeMode_ = bottom;

                if(position.x()<moveRect.position().x())
                    resizeMode_ |= left;
                else if(position.x()>moveRect.position().x() + moveRect.size().x())
                    resizeMode_ |= right;
            }
        }
    }
    else if(frameWindow_.isMovable())
    {
        handleMode_ = move;
    }
    else    
        handleMode_ = none;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







// **************************************************************************************************
//                                            FrameWindowDesc
// **************************************************************************************************





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
FrameWindowDesc::FrameWindowDesc() :
    topLeftDesc_(new DefaultFrameDesc(topLeft)),
    titleDesc_(new DefaultFrameDesc(title)),
    topDesc_(new DefaultFrameDesc(top)),
    topRightDesc_(new DefaultFrameDesc(topRight)),
    rightDesc_(new DefaultFrameDesc(right)),
    bottomRightDesc_(new DefaultFrameDesc(bottomRight)),
    bottomDesc_(new DefaultFrameDesc(bottom)),
    bottomLeftDesc_(new DefaultFrameDesc(bottomLeft)),
    leftDesc_(new DefaultFrameDesc(left)),
    textDesc_(new TextDesc(WindowManager::defaultFont(), 18, 0, 0, 0, Color(0.0f, 0.0f, 0.3f, 1.0f))),
    topBorderWidth_(30.0f),
    leftBorderWidth_(30.0f),
    rightBorderWidth_(30.0f),
    bottomBorderWidth_(30.0f),
    frameWindow_(0)
{
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
FrameWindowDesc::~FrameWindowDesc()
{
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void
FrameWindowDesc::draw(const Core::Rectangle& area) const
{
    Renderer::setCurrentColor(Color(1, 1, 1, 1));

    // ***********************************************************************
    //  draw the background
    // ***********************************************************************

    const Core::Vector2 topLeft = area.topLeft();
    const Core::Vector2 bottomRight = area.bottomRight();
    const Core::Vector2 topRight = area.topRight();
    const Core::Vector2 bottomLeft = area.bottomLeft();


    // ***********************************************************************
    //  draw the border
    // ***********************************************************************

    // get the centers of the four rounded corners
    const Core::Vector2 topLeftCenter     = topLeft + Core::Vector2(leftBorderWidth_, topBorderWidth_);
    const Core::Vector2 bottomRightCenter = bottomRight + Core::Vector2(-rightBorderWidth_, -bottomBorderWidth_);
    const Core::Vector2 bottomLeftCenter  = Core::Vector2(topLeft.x() + leftBorderWidth_, bottomRight.y() - bottomBorderWidth_);
    const Core::Vector2 topRightCenter    = Core::Vector2(bottomRight.x() - rightBorderWidth_, topLeft.y() + topBorderWidth_);

    topLeftDesc_->draw(Core::Rectangle(topLeft, topLeftCenter));
    topRightDesc_->draw(Core::Rectangle(topRight, topRightCenter));
    rightDesc_->draw(Core::Rectangle(topRightCenter, Core::Vector2(bottomRight.x(), bottomRightCenter.y())));
    bottomRightDesc_->draw(Core::Rectangle(bottomRightCenter, bottomRight));
    bottomDesc_->draw(Core::Rectangle(bottomLeftCenter, Core::Vector2(bottomRightCenter.x(), bottomRight.y())));
    bottomLeftDesc_->draw(Core::Rectangle(bottomLeftCenter, bottomLeft));
    leftDesc_->draw(Core::Rectangle(Core::Vector2(topLeft.x(), topLeftCenter.y()), bottomLeftCenter));

    textDesc_->setText(frameWindow_->text());
    Core::Vector2 textSize = textDesc_->textSize();
    Core::Vector2 barSize(textSize.x()+10.0f, topBorderWidth_);

    const float clientWidth = bottomRightCenter.x() - topLeftCenter.x();
    if(barSize.x() > clientWidth) barSize.setX(clientWidth);

    Core::Vector2 barMini = topLeft + Core::Vector2(leftBorderWidth_, 0.0f);

    titleDesc_->draw(Core::Rectangle(Core::Vector2(topLeftCenter.x(), topLeft.y()),
        Core::Vector2(topLeftCenter.x() + barSize.x(), topLeftCenter.y())));

    topDesc_->draw(Core::Rectangle(Core::Vector2(topLeftCenter.x() + barSize.x(), topLeft.y()),
        topRightCenter));


    // ***********************************************************************
    //  draw the title
    // ***********************************************************************

    textDesc_->draw(Core::Rectangle(Core::Vector2(topLeftCenter.x(), topLeft.y()),
                    Core::Vector2(topLeftCenter.x() + barSize.x(), topLeftCenter.y())));

    Renderer::setCurrentColor(Color(1, 1, 1, 1));
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void
FrameWindowDesc::serialize(Core::Archive& anArchive)
{
    RenderDesc::serialize(anArchive);
    anArchive &
        TAGGED_OBJECT(topLeftDesc) &
        TAGGED_OBJECT(titleDesc) &
        TAGGED_OBJECT(topDesc) &
        TAGGED_OBJECT(topRightDesc) &
        TAGGED_OBJECT(rightDesc) &
        TAGGED_OBJECT(bottomRightDesc) &
        TAGGED_OBJECT(bottomDesc) &
        TAGGED_OBJECT(bottomLeftDesc) &
        TAGGED_OBJECT(leftDesc) &
        TAGGED_OBJECT(textDesc) &
        TAGGED_OBJECT(topBorderWidth) &
        TAGGED_OBJECT(leftBorderWidth) &
        TAGGED_OBJECT(rightBorderWidth) &
        TAGGED_OBJECT(bottomBorderWidth);
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// *************************************************************************************************
} // namespace Gooey
