// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________
#ifndef FRAMEWINDOW__H
#define FRAMEWINDOW__H

#include "glgooey/Window.h"
#include "glgooey/RenderDesc.h"

// *************************************************************************************************
//  Forward declarations
namespace Gooey
{
    namespace Core
    {
        class Rectangle;
    }
    class FrameWindowMouseListener;
    DEFINE_SMARTPOINTER(FrameWindowDesc);
    DEFINE_SMARTPOINTER(TextDesc);
}
// *************************************************************************************************




// *************************************************************************************************
namespace Gooey
{


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! Window with title bar, border and client area
class FrameWindow : public Window
{
    // ---------------------------------------------------------------------------------------------
    //  Construction / Destruction
    // ---------------------------------------------------------------------------------------------
public:
    //! default constructor
    FrameWindow(const Core::Rectangle& rect, Window* aParent = 0, const std::string& aText = "");

    //! destructor
    virtual ~FrameWindow();


    // ---------------------------------------------------------------------------------------------
    //  Accessors
    // ---------------------------------------------------------------------------------------------
public:
    //! Returns the frame window's client window
    Window* clientWindow() const { return clientWindow_; }

    //! Sets the frame window's client window
    FrameWindow& setClientWindow(Window* aWindow)
    {
        clientWindow_ = aWindow;
        clientWindowChanged();
        return *this;
    }

    //! Returns true if the window can be moved
    bool isMovable() const;
    
    //! Returns true if the window can be resized
    bool isResizable() const;

    //! Make the window movable (hold the left mouse down on the window frame and drag)
    FrameWindow& enableMovement();

    //! Prevent the window from being moved
    FrameWindow& disableMovement();
        
    //! Make the window resizable (hold the left mouse down on the outer window frame and drag)
    FrameWindow& enableResize();

    //! Prevent the window from being resized
    FrameWindow& disableResize();


    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    //! return the rectangle including the frame
    Core::Rectangle frameRectangle() const;
    
    //! return the rectangle in which the client resides
    Core::Rectangle clientRectangle() const;
    
    //! set the size and position of the client window using a rectangle
    FrameWindow& setFrameRectangle(const Core::Rectangle& rect);

    //! set the size and position of the client window using a rectangle
    FrameWindow& setClientRectangle(const Core::Rectangle& rect);

    // Commented in base class
    virtual Core::Vector2 recommendedSize() const;    
    virtual Window& loadAppearance(Core::Archive& anArchive, const std::string& aWindowName);
    virtual bool process(const Message& message);
    virtual Window* windowBelow(const Core::Vector2& p);
    virtual bool isBelow(const Core::Vector2& aPosition);
    virtual bool doesHierarchyContain(Window* aWindow) const;



    // ---------------------------------------------------------------------------------------------
    //  Implementation
    // ---------------------------------------------------------------------------------------------
protected:
    //! render the window to the current OpenGL context
    virtual Window& draw();

    //! react to the client window changing
    virtual FrameWindow& clientWindowChanged();

    //! called when the window is resized
    virtual void onResizing(Core::Vector2& size);
    
    //! called when the window is resized (resizes the client)
    virtual void onResize();

    //! called when the window is moved (repositions the client)
    virtual void onMove();

private:
    FrameWindowMouseListener* mouseListener_;
    Window* clientWindow_;
    int captureCount_;
    bool isMovable_;
    bool isResizable_;

    friend class FrameWindowMouseListener;

    FrameWindowDescPtr renderDesc_;
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! \brief The render desc class for frame windows
class FrameWindowDesc : public RenderDesc
{
    // ---------------------------------------------------------------------------------------------
    //  Construction / Destruction
    // ---------------------------------------------------------------------------------------------
public:
    //! Default Constructor
    FrameWindowDesc();

    //! Destructor
    virtual ~FrameWindowDesc();


    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    //! sets the frame window that this desc will be drawing
    FrameWindowDesc& setFrameWindow(FrameWindow* aFrameWindow)
    {
        frameWindow_ = aFrameWindow;
        return *this;
    }

    //! draws on the passed in area according to the desc
    virtual void draw(const Core::Rectangle& area) const;

    //! Returns the border width at the top of the frame
    float topBorderWidth() const { return topBorderWidth_; }

    //! Returns the border width at the left of the frame
    float leftBorderWidth() const { return leftBorderWidth_; }

    //! Returns the border width at the right of the frame
    float rightBorderWidth() const { return rightBorderWidth_; }

    //! Returns the border width at the bottom of the frame
    float bottomBorderWidth() const { return bottomBorderWidth_; }

    // ---------------------------------------------------------------------------------------------
    //  Implementation
    // ---------------------------------------------------------------------------------------------
private:
    RenderDescPtr  topLeftDesc_;
    RenderDescPtr  titleDesc_;
    RenderDescPtr  topDesc_;
    RenderDescPtr  topRightDesc_;
    RenderDescPtr  rightDesc_;
    RenderDescPtr  bottomRightDesc_;
    RenderDescPtr  bottomDesc_;
    RenderDescPtr  bottomLeftDesc_;
    RenderDescPtr  leftDesc_;
    TextDescPtr    textDesc_;

    float topBorderWidth_;
    float leftBorderWidth_;
    float rightBorderWidth_;
    float bottomBorderWidth_;

    FrameWindow* frameWindow_;



    // ---------------------------------------------------------------------------------------------
    //  Serialization
    // ----------------------------------------------------------------------------------------------
    DECLARE_SERIALIZATION;
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


// *************************************************************************************************
} // namespace Gooey



#endif
