// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________

#include <cmath>

#include "GridLayouter.h"

#include "glgooey/Window.h"
#include "glgooey/core/Utils.h"


// *************************************************************************************************
namespace Gooey
{


REGISTER_CLASS(GridLayouter);


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
GridLayouter::GridLayouter() :
    numberOfRows_(0),
    numberOfColumns_(1),
    horizontalSpacing_(4),
    verticalSpacing_(4)
{
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
GridLayouter::GridLayouter(size_t numRows, size_t numColumns) :
    numberOfRows_(numRows),
    numberOfColumns_(numColumns),
    horizontalSpacing_(4),
    verticalSpacing_(4)
{
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
GridLayouter::GridLayouter(size_t numRows, size_t numColumns, size_t aHorizontalSpacing, size_t aVerticalSpacing) :
    numberOfRows_(numRows),
    numberOfColumns_(numColumns),
    horizontalSpacing_(aHorizontalSpacing),
    verticalSpacing_(aVerticalSpacing)
{
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
BaseLayouter&
GridLayouter::addWindow(Window* window)
{
    windows_.push_back(window);
    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
BaseLayouter&
GridLayouter::removeWindow(Window* window)
{
    windows_.remove(window);
    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
BaseLayouter&
GridLayouter::arrangeWithin(Window* parentWindow)
{
    if(windows_.size() > 0)
    {
        const Core::Vector2 childSize = computeChildSize(parentWindow);
        
        std::list<Window*>::iterator it = windows_.begin();

        const size_t numRows = actualNumberOfRows();
        float y = static_cast<float>(verticalSpacing());
        for(size_t row = 0; row < numRows; ++row)
        {
            arrangeRow(parentWindow, y, it);
            if(it == windows_.end()) break;
            y += childSize.y() + verticalSpacing();
        }
    }

    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// **************************************************************************************************
void
GridLayouter::arrangeRow(Window* parentWindow, float y, std::list<Window*>::iterator& it)
{
    const Core::Vector2 childSize = computeChildSize(parentWindow);
        
    const size_t numColumns = actualNumberOfColumns();
    float x = static_cast<float>(horizontalSpacing());
    for(size_t column = 0; column < numColumns; ++column)
    {
        Window* window = *it;
        window->setPosition(Core::Vector2(x, y) + parentWindow->position());
        window->setSize(childSize);
        x += window->size().x() + horizontalSpacing();
        ++it;
        if(it == windows_.end()) break;
    }
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







// **************************************************************************************************
size_t 
GridLayouter::actualNumberOfRows() const
{
    return (numberOfRows() == 0) ? 
        ((windows_.size() + actualNumberOfColumns()) - 1) / actualNumberOfColumns() :
        std::min(windows_.size(), numberOfRows());
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// **************************************************************************************************
size_t 
GridLayouter::actualNumberOfColumns() const
{
    size_t result = 0;
    if(numberOfRows() == 0)
    {
        result = Core::clamp(numberOfColumns(), size_t(1), windows_.size());
    }
    else
    {
        result = (numberOfColumns() == 0) ?
            ((windows_.size() + actualNumberOfRows()) - 1) / actualNumberOfRows() :
            numberOfColumns();
    }
    return result;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// **************************************************************************************************
float 
GridLayouter::totalHorizontalSpacing(size_t numColumns) const
{
    return static_cast<float>(horizontalSpacing() + (horizontalSpacing() * numColumns));
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// **************************************************************************************************
float 
GridLayouter::totalVerticalSpacing(size_t numRows) const
{
    return static_cast<float>(verticalSpacing() + (verticalSpacing() * numRows));
}
// **************************************************************************************************
    
    
    
    
    
// **************************************************************************************************
Core::Vector2 
GridLayouter::computeChildSize(Window* parentWindow) const
{
    Core::Vector2 result(0.0f, 0.0f);
    
    if(windows_.size() > 0)
    {
        const float oneOverNumRows = 1.0f / static_cast<float>(actualNumberOfRows());
        const float oneOverNumColumns = 1.0f / static_cast<float>(actualNumberOfColumns());

        result = Core::Vector2(
            (parentWindow->size().x() - totalHorizontalSpacing(actualNumberOfColumns())) * oneOverNumColumns,
            (parentWindow->size().y() - totalVerticalSpacing(actualNumberOfRows())) * oneOverNumRows);
    }
    return result;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// **************************************************************************************************
} // namespace Gooey

