// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________

#include "glgooey/MouseMessage.h"

#include "glgooey/core/Utils.h"
#include "glgooey/Window.h"
#include "glgooey/WindowManager.h"
#include "glgooey/core/Rectangle.h"





// **************************************************************************************************
namespace Gooey
{


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
MouseMessage::MouseMessage(Id anId, const Core::Vector2& aPosition) :
    Message(anId),
    position_(aPosition)
{
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
MouseClickMessage::MouseClickMessage(Id anId, const Core::Vector2& aPosition,
                                 Button aButton, Event anEvent) :
    MouseMessage(anId, aPosition),
    button_(aButton),
    event_(anEvent)
{
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// *************************************************************************************************
//                              MouseListener
// *************************************************************************************************





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
MouseListener::MouseListener() :
    oldPosition_(-1e6f, -1e6f)
{
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<










// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
bool
MouseListener::dispatchMessage(const Message& aMessage, Window* aReceiverWindow)
{
    bool ret = false;

    Message::Id id = aMessage.id();

    if( (id == Message::mouseClick)   ||
        (id == Message::mouseEntered) ||
        (id == Message::mouseExited)  ||
        (id == Message::mouseMoved) )
    {
        const MouseMessage& mouseMessage = dynamic_cast<const MouseMessage&>(aMessage);

        ret = checkForEnterOrExit(mouseMessage, aReceiverWindow);

        ret |= callHandlers(ret, mouseMessage, aReceiverWindow);        
    }
    return ret;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<










// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
bool 
MouseListener::checkForEnterOrExit(const MouseMessage& aMessage, Window* aReceiverWindow)
{
    bool result = false;
    if(aMessage.id() == Message::mouseMoved)
    {
        const bool newPosInWindow = aReceiverWindow->isBelow(aMessage.position());
        const bool oldPosInWindow = aReceiverWindow->isBelow(oldPosition_);
        
        const Window* captureWindow = WindowManager::instance().captureWindow();
        
        if((!captureWindow && newPosInWindow) || captureWindow == aReceiverWindow)
            result = moved(aMessage);

        if( oldPosInWindow && !newPosInWindow )
        {
            aReceiverWindow->process(MouseMessage(Message::mouseExited, aMessage.position()));
        }
        else if( !oldPosInWindow && newPosInWindow )
        {
            aReceiverWindow->process(MouseMessage(Message::mouseEntered, aMessage.position()));
        }
        oldPosition_ = aMessage.position();
    }
    return result;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<










// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
bool 
MouseListener::callHandlers(bool aResult, const MouseMessage& aMessage, Window* aReceiverWindow)
{
    const Core::Vector2& newPosition = aMessage.position();
    const bool newPosInWindow = aReceiverWindow->isBelow(newPosition);

    const Window* captureWindow = WindowManager::instance().captureWindow();

    bool result = aResult;
    
    // pass the message on if the mouse is in the window
    switch(aMessage.id())
    {
    case Message::mouseClick:
        if((!captureWindow && newPosInWindow) || captureWindow == aReceiverWindow)
            result = result || clicked((MouseClickMessage&)(aMessage));
        break;

    case Message::mouseEntered:
        if((!captureWindow && newPosInWindow) || captureWindow == aReceiverWindow)
            result = result || entered(aMessage);
        break;

    case Message::mouseExited:
        if(!captureWindow || captureWindow == aReceiverWindow)
            result = result || exited(aMessage);
        break;

    default:
        break;
    }
    return result;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<



// **************************************************************************************************
} // namespace Gooey


