// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________
#ifndef __GOOEY_CORE_SERIALIZATION_H__
#define __GOOEY_CORE_SERIALIZATION_H__


#include <utility>

#include "glgooey/core/StandardException.h"

// *************************************************************************************************
namespace Gooey
{


// *************************************************************************************************
namespace Core
{


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! \brief Template based class PoolInterface
template<typename T>
class PoolInterface
{
    // ----------------------------------------------------------------------------------------------
    //  Xstruction
    // ----------------------------------------------------------------------------------------------
public:
    //! Constructor
    PoolInterface()
    {
        nextElement_ = firstElement_;
        firstElement_ = (T*)this;
    }

    //! Destructor
    virtual ~PoolInterface()
    {
        T* prevElement = 0;
        T* element = firstElement_;

        while(element != 0 && element != this)
        {
            prevElement = element;
            element = element->nextElement();
        }
        if( (element != 0) && (prevElement != 0) ) prevElement->setNextElement(element->nextElement());
    }


    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
    //! Returns a pointer to the first element in the linked list
    static T* firstElement() { return firstElement_; }
    
    //! Returns a pointer to the next element in the linked list
    void setNextElement(T* anElement) { nextElement_ = anElement; }


    //! Returns a pointer to the next element in the linked list
    T* nextElement() { return nextElement_; }

private:        
    static T* firstElement_; //!< Head of the linked list of objects
    T* nextElement_;         //!< Pointer to the next item in the linked list of objects    


};



// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! \brief Template based class factory
//!
//! In order to support serialization of polymorphic types each polymorphic type must be registered
//! with the class factory this is done via a simple declaration:
//! \code
//!   Factory<MyClassType> MyFactory("MyClassType");
//! \endcode
template<typename T>
class FactoryInterface : public PoolInterface< FactoryInterface<T> >
{
    // ----------------------------------------------------------------------------------------------
    //  Xstruction
    // ----------------------------------------------------------------------------------------------
public:
    //! Constructor
    FactoryInterface(const std::string& aName) : PoolInterface< FactoryInterface<T> >(), className_(typeid(T).name()), niceName_(aName) {}


    //! Destructor
    virtual ~FactoryInterface() {}


    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    //! Creates an instance of the class with the passed in name
    static T* createClass(const std::string& aName)
    {
        FactoryInterface<T>* f = factoryByNiceName(aName);
        return f?f->create():0;
    }

    //! Returns the name of the class (as given by type_info) which can be created by this factory
    virtual std::string className() const { return className_; }    

    //! Returns a name given to the class which can be created by this factory
    virtual std::string niceName() const { return niceName_; }
    
    //! Returns the factory for the class type with the passed in name as given by RTTI
    static FactoryInterface<T>* factoryByClassName(const std::string& aName)
    {
        FactoryInterface<T>* factory = firstElement();
        while(factory != 0 && factory->className() != aName) factory = factory->nextElement();    
        return factory;
    }

    //! \brief Returns the factory for the class type with the passed in name as specified in the
    //! factory's constructor
    static FactoryInterface<T>* factoryByNiceName(const std::string& aName)
    {
        FactoryInterface<T>* factory = firstElement();
        while(factory != 0 && factory->niceName() != aName) factory = factory->nextElement();    
        return factory;        
    }

    //! Do the actual creation
    virtual T* create() const = 0;

    // ---------------------------------------------------------------------------------------------
    //  Implementation
    // ----------------------------------------------------------------------------------------------
private:
    std::string className_; //!< The name of the class which can be created by this factory
    std::string niceName_;  //!< The name given to the class on construction of the factory object
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
template<typename ROOTTYPE, typename T>
class Factory : public FactoryInterface<ROOTTYPE>
{
    // ----------------------------------------------------------------------------------------------
    //  Construction
    // ----------------------------------------------------------------------------------------------
public:
    //! Constructor
    Factory(const std::string& aName) : FactoryInterface<ROOTTYPE>(aName)  {}


    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    //! Do the actual creation
    virtual ROOTTYPE* create() const { return new T; } 
};




#define IMPLEMENT_CLASS_POOL(T) T* PoolInterface<T>::firstElement_ = 0
#define IMPLEMENT_CLASS_FACTORY(x) IMPLEMENT_CLASS_POOL(Gooey::Core::FactoryInterface<x>)

#define REGISTER_CLASS(className) Gooey::Core::Factory<className::rootClass, className> __##className##Factory__(#className)




// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! Class used to give names to objects
template<typename T>
class ObjectNamer
{
    // ----------------------------------------------------------------------------------------------
    //  Xstruction
    // ----------------------------------------------------------------------------------------------
public:
    //! Construction requires a reference to the object and the name to give the object
    ObjectNamer(const char* aName, T& anObject) : 
        pair_(aName, &anObject)        
        {}


    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    //! Returns the name of the object
    const char* name() const { return pair_.first; }

    //! Returns a reference to the object being named
    T& object() const { return *pair_.second; }

    //! reads/writes the object from/to the passed in archive
    template<typename Archive> void serialize(Archive& anArchive) { anArchive & object(); }

    // ---------------------------------------------------------------------------------------------
    //  Implementation
    // ----------------------------------------------------------------------------------------------
private:
    std::pair<const char*, T*> pair_;  //!< The pair which maps a name to an object
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<



// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
template<typename T>
ObjectNamer<T> nameObject(T& anObject, const char* aName)
{    
    return ObjectNamer<T>(aName, anObject);
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

#define TAGGED_OBJECT(x) Gooey::Core::nameObject(x##_, #x)



// *************************************************************************************************
} // namespace Core

// *************************************************************************************************
} // namespace Gooey


#include "glgooey/services/interfaces/Archive.h"


#endif
