// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________
#ifndef __SMARTPOINTER__H_
#define __SMARTPOINTER__H_



// *************************************************************************************************
namespace Gooey
{


// *************************************************************************************************
namespace Core
{




// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! \brief Macro definition for declaring a reference counting interface
#define DECLARE_REFEERENCE_COUNTING \
public: \
    void incrementReferenceCount() \
    { \
        ++referenceCount_; \
    } \
    \
    void decrementReferenceCount() \
    { \
        if ( --referenceCount_ == 0 ) \
        delete this; \
    } \
    \
    unsigned int referenceCount() const \
    { \
        return referenceCount_; \
    } \
    \
    static unsigned int objectCount() \
    { \
        return objectCount_; \
    } \
    \
private: \
    unsigned int referenceCount_; \
    static unsigned int objectCount_;
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<



//----------------------------------------------------------------------------------------
#define IMPLEMENT_REFEERENCE_COUNTING(classname) \
	unsigned int classname::objectCount_ = 0

#define IMPLEMENT_REFEERENCE_COUNTING_CONSTRUCTION \
	referenceCount_ = 0; \
	objectCount_++
//----------------------------------------------------------------------------------------
#define IMPLEMENT_REFEERENCE_COUNTING_DESTRUCTION \
	objectCount_--
//----------------------------------------------------------------------------------------
#define DEFINE_SMARTPOINTER(classname) \
	class classname; \
    typedef ::Gooey::Core::SmartPointer<classname> classname##Ptr
//----------------------------------------------------------------------------------------
#define CAST_SMARTPOINTER(type, smartpointer) ((type*)(void*)(smartpointer))



// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! \brief A template for implementing smartpointers. It supports reference counting
//and automatic object destruction to support shared objects
//----------------------------------------------------------------------------------------
template <class T> 
class SmartPointer 
{
    // ---------------------------------------------------------------------------------------------
    //  Xstruction
    // ---------------------------------------------------------------------------------------------
public:	
	SmartPointer(T* anObject = 0);
	SmartPointer(const SmartPointer& aPtr);
	~SmartPointer();

	
    // ---------------------------------------------------------------------------------------------
    //  Operators
    // ---------------------------------------------------------------------------------------------
public:    
    operator T*() const;
	T& operator*() const;
	T* operator->() const;

	SmartPointer& operator=(const SmartPointer& sref);
	SmartPointer& operator=(T* anObject);		

	bool operator==(int ptr) const;
	bool operator!=(int ptr) const;
	
	bool operator==(T* anObject) const;
	bool operator==(const T* anObject) const;
	
	bool operator!=(T* anObject) const;
	bool operator!=(const T* anObject) const;
	
	bool operator==(SmartPointer& aPtr) const;
	bool operator==(const SmartPointer& aPtr) const;

	bool operator!=(SmartPointer& aPtr) const;
	bool operator!=(const SmartPointer& aPtr) const;

		
    // ---------------------------------------------------------------------------------------------
    //  Implementation
    // ---------------------------------------------------------------------------------------------
protected:
	T* object_;
};

#include "SmartPointer.inl"

// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


// *************************************************************************************************
} // namespace Core

// *************************************************************************************************
} // namespace Gooey


#endif
