// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________
#ifndef __GOOEY_CORE_ARCHIVE_H__
#define __GOOEY_CORE_ARCHIVE_H__


#include "glgooey/core/Serialization.h"
#include "glgooey/core/StandardException.h"

#include "glgooey/services/interfaces/Archive.comp"


// *************************************************************************************************
namespace Gooey
{


// *************************************************************************************************
namespace Core
{


#define DECLARE_SERIALIZATION \
public: \
    void serialize(Gooey::Core::Archive& anArchive)


#define DECLARE_SERIALIZATION_ROOTCLASS(className) \
public: \
    typedef className rootClass; \
    virtual void serialize(Gooey::Core::Archive& anArchive)


class Archive
{
    // ----------------------------------------------------------------------------------------------
    //  Xstruction
    // ----------------------------------------------------------------------------------------------
public:
    //! Construction
    Archive();
    
    //! Destructor
    virtual ~Archive();


    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    //! Opens an archive
    virtual bool open(const std::string& aFileName, bool isReading);

    //! Closes an archive
    virtual void close() = 0;

    //! The input/output operator
    template<typename T> Archive& operator& (const ObjectNamer<T>& aNamedObject)
    {
        void* saveState = save();
            
        if(seek(aNamedObject.name()))
            serialize(resolveType(aNamedObject.object()));
        else
            onFailure(resolveType(aNamedObject.object()));
    
        restore(saveState);
        return *this;
    }

    const std::string& baseDirectory() const { return baseDirectory_; }
    
    std::string convertFileName(const std::string& aFileName) const;

    bool isReading() const { return isReadingFlag_; }


    // ---------------------------------------------------------------------------------------------
    //  Implementation
    // ----------------------------------------------------------------------------------------------
protected:
	//! \brief Read in a generic object
    template <typename T> void serialize(T& anObject asObject)
    {  
        anObject.serialize(*this);
    }

    //! Used to support serialization of object pointers
    template <typename T> void serialize(T*& anObject asPointer)
    {
        if(isReading())
        {
            determineClassName();
        
            if(!className().empty())
            {        
                FactoryInterface<typename T::rootClass>* f = FactoryInterface<typename T::rootClass>::factoryByNiceName(className());
                typename T::rootClass* tmpObject = f?f->create():0;
                if(tmpObject)
                    anObject = dynamic_cast<T*>(tmpObject);
                else
                    anObject = 0;
            }
            else
                anObject = new T;
        
            if(anObject)
                serialize(resolveType(*anObject));
        }
        else
        {
            if(anObject!=0)
            {
                FactoryInterface<typename T::rootClass>* f = FactoryInterface<typename T::rootClass>::factoryByClassName(typeid(*anObject).name());
                setClassName(f?f->niceName().c_str():0);            
                saveClassName();            
            
                anObject->serialize(*this);        
            }
        }
    }
        
#ifdef GOOEY_NO_NESTED_TEMPLATES
	
    //! Used to support serialization of SmartPointers
    template<typename T>
    void serialize(SmartPointer<T>& anObject asSmartPointer)
    {
        T* tmpObject = NULL;

        if(isReading())
        {        
            serialize(resolveType(tmpObject));        
            anObject = tmpObject;
        }
        else
        {
            T* tmpObject = anObject;
            serialize(resolveType(tmpObject));
        }
    }


#else
	
    //! Used to support serialization of object pointer wrappers    
	template<typename T, template<typename> class Wrapper>
    void serialize(Wrapper<T>& anObject asSmartPointer);

#endif

    //! Used to support serialization of STL vectors
    template<typename T> void serialize(std::vector<T>& aContainer asContainer) { serializeContainer(aContainer); }

    //! Used to support serialization of STL lists
    template<typename T> void serialize(std::list<T>& aContainer asContainer) { serializeContainer(aContainer); }

    //! Used to support serialization of STL deques
    template<typename T> void serialize(std::deque<T>& aContainer asContainer) { serializeContainer(aContainer); }

    //! Used to support serialization of STL maps
    template<typename T1, typename T2> void serialize(std::map<T1, T2>& aContainer asObject) { serializeContainer(aContainer); }

    //! Used to support serialization of std::pair objects as used in maps and multimaps
    template<typename T1, typename T2> void serialize(std::pair<const T1, T2>& aPair asPair)
    {
        *this 
            & ObjectNamer<T1>("first", aPair.first) 
            & ObjectNamer<T1>("second", aPair.second)
        ;
    }


    //! Used to support serialization of char values
    virtual void serialize(char& aChar asObject) = 0;

    //! Used to support serialization of unsigned char values
    virtual void serialize(unsigned char& anUnsignedChar asObject) = 0;

    //! Used to support serialization of integer values
    virtual void serialize(int& anInteger asObject) = 0;

    //! Used to support serialization of unsigned integer values
    virtual void serialize(unsigned int& anUnsignedInteger asObject) = 0;

    //! Used to support serialization of boolean values
    virtual void serialize(bool& aBoolean asObject) = 0;

    //! Used to support serialization of floating point values
    virtual void serialize(float& aFloat asObject) = 0;

    //! Used to support serialization of double values
    virtual void serialize(double& aDouble asObject) = 0;

    //! Used to support serialization of std::strings
    virtual void serialize(std::string& aString asObject) = 0;

    //! Derived classes should serialize here the container size or
    //! whatever information needed to serialize a Container properly
    virtual void firstElement() {}

    //! Derived classes must implement this method to signal that
    //! there is a valid element left in the archive
    virtual bool lastElement() { return true; }
    
    //! Derived classes must implement this method to prepare
    //! the serialization of the next element in the container
    virtual void nextElement() {}

    //! Method called whenever the classname is to be retrieved or to be written.    
    virtual void determineClassName() {}
    virtual void saveClassName() {}
    
    //! Sets the classname of the current object
    void setClassName(const char* aClassName);
    
    //! Returns the current classname
    const std::string& className() { return className_; }

    //! If an archive assoziates an object with a name it should implement
    //! this method to prepare the serialization of the named object.
    //! If no object with the requested name is found, false is returned.
    virtual bool seek(const char* anObjectName) { return true; }    

    //! Used to save to current state when serializing object hierarchies    
    virtual void* save() { return 0; }

    //! Used to restore the previous saved state
    virtual void restore(void*) {}


private:    
    //! Set pointer that could not be read to zero
    template<typename T>
    void onFailure(T*& aPointer asPointer) { aPointer = 0; }
        
#ifdef GOOEY_NO_PARTIAL_TEMPLATE_FUNCTION_ORDERING
    void onFailure(...) {}
#else
    template<typename T>
    void onFailure(T&) {}
#endif

    //! Generic method for serialization of STL containers
    template<typename Container> void serializeContainer(Container& aContainer)
    {
        void* saveState = save();
    
        if(isReading())
        {
            aContainer.clear();
        
            for(firstElement(); !lastElement(); nextElement())
            {
                typename Container::value_type entry;
                serialize(resolveType(entry));            
                aContainer.insert(aContainer.end(), entry);
            }
        }
        else
        {       
            if(!aContainer.empty())
            {
                firstElement();
                for(typename Container::iterator it = aContainer.begin(); it != aContainer.end(); ++it)
                {                
                
                    nextElement();
                    serialize(resolveType(*it));

                }
                lastElement();
            }

        }  

        restore(saveState);    
    }



private:    
    std::string baseDirectory_;
    std::string className_;    
    bool isReadingFlag_;
};




class ArchiveFactoryInterface : public PoolInterface<ArchiveFactoryInterface> {
    // ----------------------------------------------------------------------------------------------
    //  Datatypes
    // ----------------------------------------------------------------------------------------------
public:
    enum OperationMode { nothing = 0x00, read = 0x01, write = 0x02, readAndWrite = read | write };


    // ----------------------------------------------------------------------------------------------
    //  Xstruction
    // ----------------------------------------------------------------------------------------------
public:
    ArchiveFactoryInterface(const std::string& anArchiveDescription, const std::string anExtensionList, OperationMode anOperationMode);

    virtual ~ArchiveFactoryInterface() {}


    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    //! Creates an instance of an archive that handles the given filename
    //! with the given operation
    static Archive* createArchive(const std::string& aFileName, OperationMode anOperationMode);    

    //! Returns the factory for the class type with the passed in name as given by RTTI
    static ArchiveFactoryInterface* findArchive(const std::string& aFileName, OperationMode anOperationMode);

    //! Returns a description given to the archive which can be created by this factory
    std::string description() const { return description_; }

    //! Returns an file extension list that can be handled by the archive 
    //! which can be created by this factory
    std::string extensionList() const { return extensionList_; }

    //! Checks whether an extension can be handled or not
    bool handlesExtension(const std::string& anExtension) const;

    //! Checks whether a file can be handled or not
    bool handles(const std::string& aFileName, OperationMode anOperationMode);

    //! Do the actual creation
    virtual Archive* create() const = 0;

    // ---------------------------------------------------------------------------------------------
    //  Implementation
    // ----------------------------------------------------------------------------------------------
private:
    std::string description_;     //!< The description of the archive
    std::string extensionList_;   //!< The file extensions that can be handled by the archive
    OperationMode operationMode_; //!< The operation mode
};



template<typename T>
class ArchiveFactory : public ArchiveFactoryInterface
{
    // ----------------------------------------------------------------------------------------------
    //  Construction
    // ----------------------------------------------------------------------------------------------
public:
    //! Constructor
    ArchiveFactory(const std::string& anArchiveDescription, const std::string anExtensionList, OperationMode aOperationMode) 
        : ArchiveFactoryInterface(anArchiveDescription, anExtensionList, aOperationMode)  {}


    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    //! Do the actual creation
    virtual Archive* create() const { return new T; } 
};



#define REGISTER_ARCHIVE(className, desc, extensions, op) \
    Gooey::Core::ArchiveFactory<className> __##className##Archive__(desc, extensions, op)


#include "glgooey/services/interfaces/Archive.inl"



// *************************************************************************************************
} // namespace Core

// *************************************************************************************************
} // namespace Gooey


#endif
