// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________
#ifndef __GOOEY_CORE_XML_INPUT_ARCHIVE_H__
#define __GOOEY_CORE_XML_INPUT_ARCHIVE_H__

#include <string>
#include <vector>
#include <list>
#include <deque>
#include <map>
#include <set>

#include "tinyxml.h"
#include "glgooey/services/interfaces/Archive.h"
#include "glgooey/core/Utils.h"



// *************************************************************************************************
namespace Gooey
{



// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! Archive for reading in xml data
class XMLInputArchive : public Core::Archive
{
    // ----------------------------------------------------------------------------------------------
    //  Xstruction
    // ----------------------------------------------------------------------------------------------
public:
    //! Construction
    XMLInputArchive();

    //! Construction requires the stream the archive is to operate on
    XMLInputArchive(const std::string& aFileName);

    //! Destructor
    virtual ~XMLInputArchive() {}
    

    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    //! Opens an archive
    virtual bool open(const std::string& aFileName, bool isReading);

    //! Closes an archive
    virtual void close();


    // ---------------------------------------------------------------------------------------------
    //  Implementation
    // ----------------------------------------------------------------------------------------------
private:
    //! Used to support serialization of char values
    void serialize(char& aChar asObject);

    //! Used to support serialization of unsigned char values
    void serialize(unsigned char& anUnsignedChar asObject);

    //! Used to support serialization of integer values
    void serialize(int& anInteger asObject);

    //! Used to support serialization of unsigned integer values
    void serialize(unsigned int& anUnsignedInteger asObject);

    //! Used to support serialization of boolean values
    void serialize(bool& aBoolean asObject);

    //! Used to support serialization of floating point values
    void serialize(float& aFloat asObject);

    //! Used to support serialization of double values
    void serialize(double& aDouble asObject);

    //! Used to support serialization of std::strings
    void serialize(std::string& aString asObject);

    //! Generic method for serialization of primitive data types
    template<typename T> void readPrimitive(T& aPrimitive)
    {
        TiXmlNode* textNode = currentNode_->FirstChild();
        if(textNode != 0)
        {
            TiXmlText* text = textNode->ToText();
            if(text != 0) aPrimitive = Core::string_cast<T>(text->Value());
        }
    }

    //! Container serialization (commented in base class)
    void firstElement();
    bool lastElement();
    void nextElement();    

    //! Fetch className from XML node (commented in base class)
    void determineClassName();
    
    //! Jump to new child node (commented in base class)
    bool seek(const char* anObjectName);

    //! Returns the name for the current node if it has an attribute "name".
    //! (commented in base class)
    const char* objectName();
    
    //! State management (commented in base class)
    void* save() { return currentNode_; }
    void restore(void* aSavedNode) { currentNode_ = reinterpret_cast<TiXmlNode*>(aSavedNode); }

private:
    TiXmlDocument document_;  //!< The tinyxml document object
    TiXmlNode* currentNode_;  //!< The node that is currently being processed    
    TiXmlNode* iteratorNode_; //!< The node that is currently being iterated
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<









// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>



// *************************************************************************************************
} // namespace Gooey


#endif
