// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________




// *********************************************************************
//  a small app showing how to respond to button pressing
//  The idea is a small console with an input edit field. When the user
//  presses a button, the contents of the edit field are written
//  into a list box.
// *********************************************************************

// say windows not to use the min/max macros that conflict with the template function
// from the STL
#ifdef _WIN32
#define NOMINMAX
#endif

#include <cstdlib>
#include <iostream>
#include <GL/glut.h>

#include "glgooey/core/Rectangle.h"
#include "glgooey/core/Position.h"

#include "glgooey/WindowManager.h"
#include "glgooey/FrameWindow.h"
#include "glgooey/Button.h"
#include "glgooey/Panel.h"
#include "glgooey/EditField.h"
#include "glgooey/ComboBox.h"
#include "glgooey/ListControl.h"
#include "glgooey/TimeManager.h"
#include "glgooey/PropertyScheme.h"
#include "glgooey/FixedPosLayouter.h"
#include "glgooey/ComplexGridLayouter.h"

#include "glgooey/services/xmlArchive/XMLInputArchive.h"
#include "glgooey/services/xmlArchive/XMLOutputArchive.h"

#include "glgooey/services/glutInterface/GlutInterface.h"

using namespace Gooey;

int main_window;
GlutInterface glutInterface;



// **************************************************************************************************
//  The main GLGooey window is a moveable frame window with an edit field, a list box and a
// "clear" button in it.
//  Notice how the child windows can be created in the constructor's initialization list, and the
// "Parent" parameter is just set to 0 (as in editField_(0, "EditField") ). We can do this because
// the child<->parent relationships are set up later on when the panel is set as the frame window's
// client window and the other windows are added to the panel. When the panel is set as the frame
// window's client window, this also resizes the panel to fit just inside the frame window, which
// explains why the window rectangle passed to the panel's constructor needn't make sense.
// **************************************************************************************************
class Console : public FrameWindow
{
public:
    Console() :
        FrameWindow(Core::Rectangle(0, 0, 600, 600), 0, "Console - resize me by dragging my border"),
        panel_(Core::Rectangle(), 0),
        listControl_(0, Core::Rectangle(), 3),
        editField_(0, ""),
        comboBox_(0, Core::Rectangle()),
        clearButton_(0, "Clear")
    {
        Core::Rectangle clientRect = clientRectangle();
        
        // Leave this out and the user will not be able to drag the window around by its frame
        enableMovement();
        enableResize();

        // set the edit field's size
        editField_.setSize(Core::Vector2(clientRectangle().width()-202, 30));
        comboBox_.setSize(Core::Vector2(100, 30));        
        
        std::vector<float> widths;
        std::vector<float> heights;
        
        std::vector<std::string> strings(3);
        strings[0] = "Column1";
        strings[1] = "Column2";
        strings[2] = "Column3";        
        listControl_.addRow(strings);        
        
        strings[0] = "--------------";
        strings[1] = "--------------";
        strings[2] = "--------------";
        listControl_.addRow(strings);
        
        strings[0] = "When adding a";
        strings[1] = "the number of columns";
        strings[2] = "All data of this";  
        listControl_.addRow(strings);
        
        strings[0] = "new value via the";
        strings[1] = "will be set to 2.";
        strings[2] = "column will be lost";  
        listControl_.addRow(strings);
        
        strings[0] = "edit field,";        
        strings[1] = "";
        strings[2] = "";
        listControl_.addRow(strings);

        // wire up the signal/slot connections - pressing Return in the edit field should have
        // the same effect as pressing the clear button
        clearButton_.pressed.connect(this, &Console::clearButtonPressed);
        editField_.returnPressed.connect(this, &Console::returnPressed);        
        
        //panel_.setLayouter(new GridLayouter(4, 1, 5, 5));
        
        widths.clear();
        ComplexGridLayouter* layouter = new ComplexGridLayouter;
        panel_.setLayouter(layouter);
                
        widths.push_back(0);
        widths.push_back(150);
        widths.push_back(85);
        layouter->setColumnWidths(widths);
        
        heights.push_back(0);
        heights.push_back(30);
        layouter->setRowHeights(heights);        
        
        ComplexGridCellInfo info;        
        info.columnIndex = 0; info.rowIndex = 0; info.horizontalSpan = 3;        
        panel_.addChildWindow(&listControl_, info);
        
        info.columnIndex = 0; info.rowIndex = 1; info.horizontalSpan = 1;
        panel_.addChildWindow(&editField_, info);
        
        info.columnIndex = 1; info.rowIndex = 1; info.horizontalSpan = 1;
        panel_.addChildWindow(&comboBox_, info);
        
        info.columnIndex = 2; info.rowIndex = 1; info.horizontalSpan = 1;
        panel_.addChildWindow(&clearButton_, info);        
                
        // put the panel in the frame window
        setClientWindow(&panel_);
    }


public:
    // this is the slot that responds to the button being pressed
    void clearButtonPressed()
    {
      listControl_.removeAllRows();
      comboBox_.removeAllStrings();      
    }
        
    void returnPressed()
    {
        // get the edit field's text, add it to the list box, then clear the edit field
        std::string str = editField_.text();
        if(str != "") {
          listControl_.setNumberOfColumns(2);
        
          std::vector<std::string> strings;
          strings.push_back(str);
          strings.push_back("<NULL>");
          listControl_.addRow(strings);
          
          comboBox_.addString(str);
        }
        editField_.setText("");
    }

private:
    // the controls
    Panel         panel_;
    EditField     editField_;
    ListControl   listControl_;
    ComboBox      comboBox_;
    Button        clearButton_;
};



class MyPanel : public Panel {
public:
      MyPanel(Window* aParent = NULL, const std::string& aText = "")
        : Panel(aParent, aText)
      {  
          XMLInputArchive archive("../../samples/data/example5.xml");
      
          archive & Core::ObjectNamer<Panel>("mainPanel", *this);
      
          // !!!HACK to force linking the FixedPosLayouter class
          FixedPosLayouter l;
                    
          archive & Core::ObjectNamer<Panel*>("centerPanel", mainPanel_);

          XMLOutputArchive opArchive("test.xml");
                    
          opArchive & Core::ObjectNamer<Panel*>("centerPanel", mainPanel_);

          leftBarLabel_ = new StaticText(mainPanel_, "");
          archive & Core::ObjectNamer<StaticText>("sideBar", *leftBarLabel_);
          rightBarLabel_ = new StaticText(mainPanel_, "");
          archive & Core::ObjectNamer<StaticText>("sideBar", *rightBarLabel_);
          
          ComplexGridLayouter* layouter = new ComplexGridLayouter(Core::Vector2(0,0));
          setLayouter(layouter);
                
          std::vector<float> floatVec;
                
          floatVec.push_back(0);
          floatVec.push_back(mainPanel_->size().x());
          floatVec.push_back(0);
          layouter->setColumnWidths(floatVec);
        
          floatVec.clear();
          floatVec.push_back(mainPanel_->size().y());
          layouter->setRowHeights(floatVec);
          
          ComplexGridCellInfo info;          
          info.horizontalSpacing = info.verticalSpacing = 0;          
          
          info.columnIndex = 0; info.rowIndex = 0; 
          addChildWindow(leftBarLabel_, info);
          
          info.columnIndex = 1; info.rowIndex = 0; 
          addChildWindow(mainPanel_, info);
          
          info.columnIndex = 2; info.rowIndex = 0; 
          addChildWindow(rightBarLabel_, info);
          
          setSize(Core::Vector2(0, mainPanel_->size().y()));
      }
      
      ~MyPanel()
      {
          delete leftBarLabel_;
          delete rightBarLabel_;
          
          delete mainPanel_;
      }
       
      Window* windowBelow(const Core::Vector2& p)
      {
          Window* ret = Panel::windowBelow(p);
          if(ret == this) return NULL;
          return ret;
      }
          
private:
    Panel* mainPanel_;    
        
    StaticText* leftBarLabel_;
    StaticText* rightBarLabel_;
};


// This has been made global for simplicity!
Console* console = 0;


// **************************************************************************************************
//  redraw in idle time - there's nothing GLGooey related here
// **************************************************************************************************
void myGlutIdle( void )
{
    if ( glutGetWindow() != main_window )
        glutSetWindow(main_window);

    glutPostRedisplay();
}


// **************************************************************************************************
//   this is where most of the integration work is done
//  myGlutMouse, myGlutMotion, myGlutKey, myGlutSpecial all send
//  their respective messages on to the window manager
// **************************************************************************************************
void myGlutMouse(int button, int state, int x, int y)
{
    glutInterface.onMouseButton(button, state, x, y);
}


void myGlutMotion(int x, int y)
{
    glutInterface.onMouseMove(x, y);    
}


void myGlutKeyDown(unsigned char key, int GOOEY_UNUSED(x), int GOOEY_UNUSED(y))
{    
    glutInterface.onKeyDown(key);
}

void myGlutKeyUp(unsigned char key, int GOOEY_UNUSED(x), int GOOEY_UNUSED(y))
{
    glutInterface.onKeyUp(key);
}

void myGlutSpecialDown(int key, int GOOEY_UNUSED(x), int GOOEY_UNUSED(y))
{
    glutInterface.onSpecialKeyDown(key);    
}

void myGlutSpecialUp(int key, int GOOEY_UNUSED(x), int GOOEY_UNUSED(y))
{
    glutInterface.onSpecialKeyUp(key);
}





// **************************************************************************************************
//   We're just going to draw a wireframe sphere in the center of the screen
//  so when a resize happens we reset the projection - not forgetting to
//  let the window manager know that the window it's running in has been
//  resized.
//   NOTE: One of the most common errors for people starting out with GLGooey is to
// forget to inform the window manager when the application window is resized!
// **************************************************************************************************
void myGlutReshape( int x, int y )
{
    WindowManager::instance().applicationResized(x, y);

    glViewport( 0, 0, x, y );

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(90, 1, 1, 100);
    gluLookAt(2, 2, 2, 0, 0, 0, 0, 1, 0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    glutPostRedisplay();
}




// **************************************************************************************************
//  the once per frame update
// **************************************************************************************************
void myGlutDisplay( void )
{
    // clear the screen
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    // draw a wireframe cone
    //glLoadIdentity();
    //glRotatef(TimeManager::systemTime()*10.0f, 1, 0, 0);
    //glColor4f( 1, 1, 1, 1 );

    // this must be called once per frame
    WindowManager::instance().update();

    glutSwapBuffers();
}



// **************************************************************************************************
//   Because GLUT does not continue the execution of main() when the application window is
//  closed we have to use the atexit callback and do our cleaning up in the corresponding function.
// **************************************************************************************************
void shutDown()
{
    WindowManager::instance().removeWindow(console);

    delete console;

    WindowManager::instance().terminate();
}


// **************************************************************************************************
//  main
// **************************************************************************************************
int main(int argc, char* argv[])
{
    atexit(shutDown);

    // **********************************************************************************************
    //   Initialize GLUT and create window
    // **********************************************************************************************
    glutInit(&argc, argv);
    glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );
    glutInitWindowPosition( 50, 50 );
    glutInitWindowSize( 1024, 768 );

    main_window = glutCreateWindow( "Gooey Example1" );

    glutDisplayFunc ( myGlutDisplay );
    glutReshapeFunc ( myGlutReshape );
    glutIdleFunc    ( myGlutDisplay );
    glutMouseFunc   ( myGlutMouse   );
    glutKeyboardFunc( myGlutKeyDown     );
    glutSpecialFunc ( myGlutSpecialDown );
    glutKeyboardUpFunc( myGlutKeyUp     );
    glutSpecialUpFunc ( myGlutSpecialUp );
    glutPassiveMotionFunc ( myGlutMotion  );
    glutMotionFunc  ( myGlutMotion  );

    glClearColor( 0, 0.2f, 0.2f, 1.0f );


    try
    {
        // **********************************************************************************************
        //  Initialize GLGooey
        // **********************************************************************************************        
        WindowManager::instance().initialize("../../samples/data/accid___.ttf", &glutInterface);
        WindowManager::instance().propertyScheme().save("blafasel.xml");
    
        // **********************************************************************************************
        //   Create an instance of the GLGooey window defined in the Console class and add it to the
        //  window manager. Once it has been added it will be drawn every time
        //  WindowManager::instance().update() is called
        // **********************************************************************************************   
        console = new Console;
        MyPanel* panel = new MyPanel;    
        
        WindowManager::instance().addWindow(console);    
        WindowManager::instance().addWindow(panel);
        
        Dock d;
        d.setDocked(true);
        d.setStretch(true, false);
        d.setVerticalAlignment(Core::Alignment::bottom);
        
        WindowManager::instance().setDock(panel, d);
    
        // **********************************************************************************************
        //  Start the main loop
        // **********************************************************************************************
        glutMainLoop();
    }
    catch(std::exception& e)
    {
        std::cout << "Exception: " << e.what() << std::endl;
        std::cin.get();
    }

    return 0;
}
